/*$Header: ansic.l,v 4.0 94/02/01 20:29:46 tedd Exp $*/

%{
/******************************************************************
	Module Name: ansic.l

	Standard ANSI C Scanner
	Reference Book: Third Edition of
		"C - A reference Manual"
		by Samuel P. Harbison & Guy L. Steele Jr.

	Programmer: Xiaokang Yu
	Date: March 2, 1992
******************************************************************/

#include <stdio.h>

/*Lowther93: Minimal detected length identified as an 'indent'*/
#define INDENT_LENGTH 2
/* TED '94--Used in loc_check to help count union lines. */
#define CODE 0
#define COM 1

char yyidentifier[80];
int num_of_preprocessor_directives = 0;
int IS_PROCEDURE_CALL = 0;

%}

letter			[A-Za-z]
digit			[0-9]
underscore		"_"
following_character	({letter}|{digit}|{underscore})
identifier		({letter}|{underscore}){following_character}*

nonzero_digit		[1-9]
octal_digit		[0-7]
hex_digit		[0-9a-fA-F]
long_suffix		[lL]
unsigned_suffix		[uU]
integer_suffix		({long_suffix}{unsigned_suffix}?|{unsigned_suffix}{long_suffix}?)
decimal_constant	({nonzero_digit}{digit}*)
octal_constant		(0{octal_digit}*)
hexadecimal_constant	(0[xX]{hex_digit}+)
integer_constant	(({decimal_constant}|{octal_constant}|{hexadecimal_constant}){integer_suffix}?)

sign_part		[+-]
floating_suffix		[fFlL]
digit_sequence		({digit}+)
dotted_digits		(({digit_sequence}"."{digit_sequence}?)|("."{digit_sequence}))
exponent		([eE]{sign_part}?{digit_sequence})
floating_constant	((({digit_sequence}{exponent})|({dotted_digits}{exponent}?)){floating_suffix}?)

character_escape_code	[ntbrfv\\'"a?]
octal_escape_code	({octal_digit}{1,3})
hex_escape_code		(x{hex_digit}+)
escape_code		({character_escape_code}|{octal_escape_code}|{hex_escape_code})
escape_character	(\\{escape_code})

c_char			([^'\\\n]|{escape_character})
c_char_sequence		({c_char}+)
character_constant	(\L?\'{c_char_sequence}\')

s_char			([^"]|{escape_character})
s_char_sequence		({s_char}+)
string_constant		\L?\"{s_char_sequence}?\"

constant		{integer_constant}|{floating_constant}|{character_constant}

whitespace_blank	[ ]
whitespace_tab		[\t]
whitespace_vertical	[\f\v]
newline			[\n]
external_file_call	"#include"[ \t]+"\""

%%

"auto"			{ loc_check(CODE); return AUTO; }
"break"			{ loc_check(CODE); add_operator("BREAK"); return BREAK; }
"case"			{ loc_check(CODE); count_keywords(); add_operator("CASE"); 
			  return CASE; }
"char"			{ loc_check(CODE); return CHAR; }
"const"			{ loc_check(CODE); return CONST; }
"continue"		{ loc_check(CODE); add_operator("CONTINUE"); return CONTINUE; }
"default"		{ loc_check(CODE); count_keywords(); add_operator("DEFAULT"); 
			  return DEFAULT; }
"do"			{ loc_check(CODE); add_operator("DO"); return DO; }
"double"		{ loc_check(CODE); return DOUBLE; }
"else"			{ loc_check(CODE); add_operator("ELSE"); return ELSE; }
"enum"			{ loc_check(CODE); find_struct = 1; return ENUM; }
"extern"		{ loc_check(CODE); return EXTERN; }
"float"			{ loc_check(CODE); return FLOAT; }
"for"			{ loc_check(CODE); count_keywords(); add_operator("FOR"); 
			    in_exp = 1; return FOR; }
"goto"			{ loc_check(CODE); count_keywords(); add_operator("GOTO");
			  return GOTO; }
"if"			{ loc_check(CODE); count_keywords(); add_operator("IF"); 
			    in_exp = 1; return IF; }
"int"			{ loc_check(CODE); return INT; }
"long"			{ loc_check(CODE); return LONG; }
"RCSID"			{ loc_check(CODE); macro_call(); }
"register"		{ loc_check(CODE); return REGISTER; }
"return"		{ loc_check(CODE); add_operator("RETURN"); return RETURN; }
"short"			{ loc_check(CODE); return SHORT; }
"signed"		{ loc_check(CODE); return SIGNED; }
"sizeof"		{ loc_check(CODE); add_operator("SIZEOF"); return SIZEOF; }
"static"		{ loc_check(CODE); return STATIC; }
"struct"		{ loc_check(CODE); find_struct = 1; return STRUCT; }
"switch"		{ loc_check(CODE); count_keywords(); add_operator("SWITCH"); 
			    in_exp = 1; return SWITCH; }
"typedef"		{ loc_check(CODE); return TYPEDEF; }
"union"			{ loc_check(CODE); find_struct = 1; return UNION; }
"unsigned"		{ loc_check(CODE); return UNSIGNED; }
"void"			{ loc_check(CODE); return VOID; }
"volatile"		{ loc_check(CODE); return VOLATILE; }
"while"			{ loc_check(CODE); count_keywords(); add_operator("WHILE"); 
			    in_exp = 1; return WHILE; }

{identifier}		{ loc_check(CODE); return(check_type()); }
{constant}		{ loc_check(CODE); change_operand_tree(yytext); 
                          return CONSTANT; }
{string_constant}	{ loc_check(CODE); change_operand_tree(yytext); return STRING; }

"+="			{ loc_check(CODE); add_operator("+="); return ASSIGN_OP; }
"-="			{ loc_check(CODE); add_operator("-="); return ASSIGN_OP; }
"*="			{ loc_check(CODE); add_operator("*="); return ASSIGN_OP; }
"/="			{ loc_check(CODE); add_operator("/="); return ASSIGN_OP; }
"%="			{ loc_check(CODE); add_operator("%="); return ASSIGN_OP; }
"<<="			{ loc_check(CODE); add_operator("<<="); return ASSIGN_OP; }
">>="			{ loc_check(CODE); add_operator(">>="); return ASSIGN_OP; }
"&="			{ loc_check(CODE); add_operator("&="); return ASSIGN_OP; }
"^="			{ loc_check(CODE); add_operator("^="); return ASSIGN_OP; }
"|="			{ loc_check(CODE); add_operator("|="); return ASSIGN_OP; }

"=="			{ loc_check(CODE); add_operator("=="); return EQUALITY_OP; }
"!="			{ loc_check(CODE); add_operator("!="); return EQUALITY_OP; }

"<"			{ loc_check(CODE); add_operator("<"); return RELATIONAL_OP; }
">"			{ loc_check(CODE); add_operator(">"); return RELATIONAL_OP; }
"<="			{ loc_check(CODE); add_operator("<="); return RELATIONAL_OP; }
">="			{ loc_check(CODE); add_operator(">="); return RELATIONAL_OP; }

"<<"			{ loc_check(CODE); add_operator("<<"); return SHIFT_OP; }
">>"			{ loc_check(CODE); add_operator(">>"); return SHIFT_OP; }

";"			{ loc_check(CODE); add_operator(";"); statement(); return ';'; }
","		 	{ loc_check(CODE); add_operator(","); return ','; }
"{"			{ loc_check(CODE); add_operator("{"); check_space(); 
                          return '{'; }
"}"			{ loc_check(CODE); return '}'; }
"("			{ loc_check(CODE); add_operator("("); p_cnt ++; return '('; }
")"			{ loc_check(CODE); if ((--p_cnt) == 0) in_exp = 0; return ')'; }
"*"			{ loc_check(CODE); add_operator("*"); return '*'; }
"/"			{ loc_check(CODE); add_operator("/"); return '/'; }
"%"			{ loc_check(CODE); add_operator("%"); return '%'; }
"["			{ loc_check(CODE); gab = 1; add_operator("["); return '['; }
"]"			{ loc_check(CODE); gab = 0; return ']'; }
":"			{ loc_check(CODE); add_operator(":"); return ':'; }
"="			{ loc_check(CODE); add_operator("="); return '='; }
"?"			{ loc_check(CODE); count_keywords(); add_operator("?"); 
                          return '?'; }
"-"			{ loc_check(CODE); add_operator("-"); return '-'; }
"+"			{ loc_check(CODE); add_operator("+"); return '+'; }
"!"			{ loc_check(CODE); add_operator("!"); return '!'; }
"~"			{ loc_check(CODE); add_operator("~"); return '~'; }
"&"			{ loc_check(CODE); add_operator("&"); return '&'; }
"|"			{ loc_check(CODE); add_operator("|"); return '|'; }
"."			{ loc_check(CODE); add_operator("."); return '.'; }
"^"			{ loc_check(CODE); add_operator("^"); return '^'; }
"++"			{ loc_check(CODE); add_operator("++"); return INCREMENT; }
"--"			{ loc_check(CODE); add_operator("--"); return DECREMENT; }
"||"			{ loc_check(CODE); add_operator("||"); count_keywords(); 
                          return OR; }
"&&"			{ loc_check(CODE); add_operator("&&"); count_keywords(); 
                          return AND; }
"->"			{ loc_check(CODE); add_operator("->"); 
                          return INDIRECT_SELECTION; }
"..."			{ loc_check(CODE); return ELIPSIS; }

"\\"			{ ; }

"/*"			{ loc_check(COM); comment_process(); }
"#"			{ loc_check(CODE); definitions(); }
{external_file_call}	{ external_file(); definitions(); }


{whitespace_blank}	{ blank(); }
{whitespace_tab}	{ horizontal_spacing(); }
{whitespace_vertical}	{ vertical_spacing(); }
{newline}		{ newline(); }

'\026'			{ ; }
'\013'			{ ; }

.			{ fprintf(stderr,"line %d: illegal character %s\n", yylineno, yytext); }

%%

/*    comment

      called by : lex.
      calls     : comment_lines in ps-code

      This procedure removes and counts the comment lines from the input
      code.

*/

/*  TED '94--rewrote comment_process so it doesn't count
 *	     blanks lines in a comment, only comment lines
 *           with text on them.
 */

comment_process()
{
  char c, c1;
  int lines = 0;     /* number of lines in comment */
  int empty_line = 0; /* flag to show when comment line is empty */
  
  while (c = input()) 
  {

    if(!(isspace(c)))
    {
      /* line busy (has something on it) */
      empty_line = 0;

      /* check to see if comment is ended */
      if(c == '*')
      {
	c1 = input();

	if(c1 == '/')
	  break;
	else
	  unput(c1);
      }
    }


    /* found a new comment line */
    if(c == '\n')
    {
      /* increment line count if there is something on this line */
      if(!empty_line)
        lines++;

      /* reset busy line flag */
      empty_line = 1;
    }

  }

  comment_lines(lines + 1);
}

/*---------------------------------------------------------------------
|                                                                     |
| Module Name: last_comment_check                      date: 1/31/94  |
|                                                                     |
| Programmer:  Dan Ash                    last modification: 1/31/94  |
|                                                                     |
| Description: checks for comments following the closing '}' of a     |
|              module.                                                |
|____________________________________________________________________*/

void last_comment_check(void)
{
  char current, next;

  while (isspace(current = input()) && current != '\n');

  if (current != '/')   /* if not a / then no comments */
  {
    unput(current);     /* replace input to stream */
    return;
  }
  else
  {
    next = input();     /* if not followed by a * then no comment */
    if (next != '*')
    {
      unput(next);      /* replace both to stream */
      unput(current);
      return;
    }
  }

  loc_check(COM);       /* found comment */
  comment_process();
} 

/*     definitions

       With a preprocessor directive, this procedure will read until
       a newline is found or a back slash.  If a back slash is encountered
       the nest line will be considered as a part of this line.

*/

definitions()
{
  char c;
  int count=0;
  
 loop:	for(c=input(); !(c=='\n' && count==0) ; c=input())
   switch(c)
   {
     case '\\': count = 1; break;
     default: count = 0; break;
   }
  num_of_preprocessor_directives ++;
  unput('\n');
}


/*     macro_call

       called by : lex.

       This procedure removes the macro RCSID 
*/

macro_call()
{
  char c;
  while( (c=input()) != '\n' );
  unput( '\n' );
}


/*     type_in

       called by : check_type

       This procedure adds a new type name into the array
*/


type_in(s)
char *s;
{
  if( tn_count > 256 )
    fprintf(stderr, "***** error typedef array is full *****\n");
  strcpy(TN[tn_count++].type_name, s);
  return;
}

/*      check_type 

        called by : lex.
        returns   : identifier type.

        This procedure takes an identifier and with the next two non white
        characters decides if the identifier is an identifier or a type
        name.  This procedure will also see if the identifier is the start
        of a procedure.
*/

check_type()
{
  int i;
  char c, c1;
  int rets = 0, rets1 = 0;
  
  strcpy( yyidentifier, yytext );
  
  while (isspace(c = input()))
    if( c== '\n')
      rets++;
  i = 0;
  while (isspace(c1 = input()))
  {
    i++;
    if( c1=='\n')
      rets1++;
  }
  unput(c1);
  while(rets1>0)
  {
    rets1--;
    unput('\n');
  }
  if (i > 0)
    unput(' ');
  unput(c);
  while(rets>0)
  {
    rets--;
    unput('\n');
  }
  unput(' ');
  
  if(c == '(')
    if(where_in_code == WHERE_GLOBAL)
      start_proc();
    else
    {
      if(where_in_code == WHERE_CODE)
        statement_list_identifiers(); 
      IS_PROCEDURE_CALL++;
    }


  for(i = 0; i < tn_count; i++)
    if(strcmp(TN[i].type_name, yytext) == 0)
      return(TYPEDEF_NAME);
  if (find_struct == 1) {
    find_struct = 0;
    return(IDENTIFIER);
  }
  
  if ((c == '_' || isalpha(c)) || ((c == '*') && (c1 == ')'))) {
    type_in(yytext);
    return(TYPEDEF_NAME);
  }
  else {
    if ((in_stmt == 1) && (c == ')') && (isalnum(c1) || (c1 == '_'))) {
      if (in_exp == 0) {
        type_in(yytext);
        return(TYPEDEF_NAME);
      }
      else
        return(IDENTIFIER);
    }
    else {
      if ((c == '*') && (isalnum(c1))) {
        if (in_stmt == 1)
          return(IDENTIFIER);
        else {
          type_in(yytext);
          return(TYPEDEF_NAME);
        }
      }
      else
        return (IDENTIFIER); 
    }
  }
}

/*     newline

       called by : lex.
       calls     : indentation, vertical_spacing, and lines_of_code.  All
       of which are in ps-code.

       This procedure reads in the white space at the start of a new line
       and checks for indentation and vertical_spacing.  The first non-
       white space character that is found will increment the number of
       lines of code for the procedure.
*/

#include <ctype.h>

/* Lowther93: Check for indention on a line starting with a '{'*/

check_space()
{
  int space = 0;
  int c;
  int newline_flag = 0;
  
  while(isspace(c = input()))
  {
    space++;
    if (c == '\t') space = INDENT_LENGTH;  /* a tab is always big enough */

    /* DEA - 93: if it finds a newline character then set the newline flag */

    if(c == '\n')
      newline_flag = 1;
  }
  unput(c);

  /* DEA - 93: if newline character is true, then increment lines of code */

  if(newline_flag) 
    lines_of_code();

  if(space >= INDENT_LENGTH) indentation();
}

newline()
{
  int space = 0;
  int c;
  int eof_flag = 0;
  
  for(c = input(); isspace(c); c = input())
  {
    switch( c )
    {
    case '\t': indentation();
      break;
    case ' ': space++;  /* lowther93: count up spaces for indentation check */
      break;
    case '\n':
    case '\v':
    case '\f': vertical_spacing();
      break;
    }
  }

  /* DEA - 93: If character is eof character, then set eof flag to 1 */

  if(c == 0)
    eof_flag = 1;
  else
    unput(c);                       /* Lowther93: Drop the last detected space
                                     character from the above case
                                     */

  /* Lowther93: The indent space is set here, it should be a #define. */
  
  if( space >= INDENT_LENGTH ) indentation();
  
  /* DEA - 93:  if eof flag NOT set then increment lines of code */

  if(!eof_flag)
    lines_of_code();
}

/*    blank

      called by : lex.
      calls     : horizontal_spacing.

      This procedure is called when a white space is found in the middle
      of a line of code.  It then checks for two or more white space
      within the code.  This check is for horizontal_spacing.
*/

blank()
{
  char c;
  if((c = input()) == ' ')
    horizontal_spacing();
  else
    unput(c);
}
