/*$Header: ansic.y,v 4.0 94/02/01 20:29:52 tedd Exp $*/

%{
/******************************************************************
	Module Name: ansic.y

	Standard ANSI C Parser from Second Edition of
	"The C Programming Language"
	by Brian W. Kernighan & Dennis M. Ritchie

	Programmer: Xiaokang Yu
	Date: March 2, 1992
******************************************************************/
#define ERROR(x) yywhere(), fputs(x,stderr);
#include "where_in_code.h"

int where_in_code = WHERE_GLOBAL;

%}

%token AUTO BREAK CASE CHAR CONST CONTINUE DEFAULT DO DOUBLE ELSE
%token ENUM EXTERN FLOAT FOR GOTO IF INT LONG REGISTER RETURN SHORT
%token SIGNED SIZEOF STATIC STRUCT SWITCH TYPEDEF UNION UNSIGNED
%token VOID VOLATILE WHILE IDENTIFIER CONSTANT STRING ASSIGN_OP
%token EQUALITY_OP RELATIONAL_OP SHIFT_OP INCREMENT DECREMENT
%token OR AND INDIRECT_SELECTION ELIPSIS TYPEDEF_NAME

%start translation_unit
%%

translation_unit
	: external_definition
	| translation_unit external_definition
	;

external_definition
	: function_definition  { in_stmt = 0; }
	| declaration
	;

function_definition
	: declarator compound_statement
	| declaration_specifiers declarator compound_statement
	| declarator { start_parm_def(); } declaration_list compound_statement
	| declaration_specifiers declarator { start_parm_def(); }
	    declaration_list compound_statement
	;

declaration
	: declaration_specifiers ';'
            {
            if(flag_23 == 1) {
              flag_23 = flag_16 = flag_13 = 0;
              flag_47 = flag_66 = 0;
              }
            yyerrok; }
	| declaration_specifiers init_declarator_list ';' 
	    {
	    if(where_in_code==WHERE_PROTO) prototype();
            if(flag_23 == 1) {
              flag_23 = flag_16 = flag_13 = 0;
              flag_47 = flag_66 = 0;
              }
            yyerrok; }
	| error ';'
		{ ERROR("declaration: error ;"); }
	;

declaration_list
	: declaration
	| declaration_list declaration
	;

declaration_specifiers
	: storage_class_specifier declaration_specifiers
            { if(flag_16 == 1) flag_13 = 1; }
	| storage_class_specifier
	| type_specifier declaration_specifiers
	| type_specifier
            { if(flag_23 == 1) flag_16 = 1; }
	| type_qualifier declaration_specifiers
	| type_qualifier
	;

storage_class_specifier
	: AUTO
	| REGISTER
	| STATIC
	| EXTERN
	| TYPEDEF
            { if(flag_23 == 0) flag_23 = 1; }
	;

type_specifier
	: VOID
	| CHAR
	| SHORT
	| INT
	| LONG
	| FLOAT
	| DOUBLE
	| SIGNED
	| UNSIGNED
	| TYPEDEF_NAME
	| struct_or_union_specifier
	| enum_specifier
	;

type_qualifier
	: CONST
	| VOLATILE
	;

struct_or_union_specifier
	: struct_or_union IDENTIFIER '{' struct_declaration_list '}'
	| struct_or_union '{'  { find_struct = 0; } struct_declaration_list '}'
	| struct_or_union IDENTIFIER
		{ yyerrok; }
	| struct_or_union error
		{ ERROR("struct_or_union_specifier: struct_or_union error"); }
	| struct_or_union '{' error '}'
		{ ERROR("struct_or_union_specifier: struct_or_union { error }"); }
	| struct_or_union IDENTIFIER '{' error '}'
		{ ERROR("struct_or_union_specifier: struct_or_union IDENTIFIER { error }"); }
	;

struct_or_union	
	: STRUCT
	| UNION
	;

struct_declaration_list
	: struct_declaration
	| struct_declaration_list struct_declaration
	;

init_declarator_list
	: init_declarator
            { if(flag_47 == 1) {
                flag_16 = flag_13 = 1;
                flag_66 = flag_47 = 0;
                }
              }
	| init_declarator_list ','
	    { if(where_in_code==WHERE_PROTO) prototype(); } 
	  init_declarator
            { if(flag_47 == 1) {
                flag_16 = flag_13 = 1;
                flag_66 = flag_47 = 0;
                }
              }
	| error
		{ ERROR("init_declarator_list: error"); }
	;

init_declarator	
	: declarator
            { if(flag_66 == 1) flag_47 = 1; }
	| declarator '=' { initialize_id(); } initializer
            { if(flag_66 == 1) flag_47 = 1; yyerrok; }
	| declarator '=' error
		{ ERROR("init_declarator: declarator = error"); }
	;

struct_declaration
	: specifier_qaulifier_list struct_declarator_list ';'
		{ yyerrok; }
	;

specifier_qaulifier_list
	: type_specifier specifier_qaulifier_list
	| type_specifier
	| type_qualifier specifier_qaulifier_list
	| type_qualifier
	;

struct_declarator_list
	: struct_declarator_list ',' struct_declarator
	| struct_declarator
	;

struct_declarator
	: declarator ':' constant_expression
	| ':' constant_expression
	| declarator
		{ yyerrok; }
	| error
		{ ERROR("struct_declarator: error"); }
	| declarator error
		{ ERROR("struct_declarator: declarator error"); }
	| declarator ':' error
		{ ERROR("struct_declarator: declarator : error"); }
	;

enum_specifier
	: ENUM enum_identifier '{' enumerator_list '}'
	| ENUM '{'  { find_struct = 0; } enumerator_list '}'
	| ENUM enum_identifier
		{ yyerrok; }
	| ENUM enum_identifier '{' error '}'
		{ ERROR("enum_specifier: ENUM IDENTIFIER { error }"); }
	| ENUM '{' error '}'
		{ ERROR("enum_specifier: ENUM { error }"); }
	| ENUM error
		{ ERROR("enum_specifier: ENUM error"); }
	;

enum_identifier
	: IDENTIFIER { add_to_list(); initialize_id(); }
	;

enumerator_list
	: enumerator_list ',' enumerator
	| enumerator
	| enumerator_list ',' error
		{ ERROR("enumerator_list: enumerator_list , error"); }
	;

enumerator
	: IDENTIFIER '=' constant_expression
	| IDENTIFIER
		{ /* PWO */ add_to_list(); yyerrok; }
	| IDENTIFIER error
		{ ERROR("enumerator: IDENTIFIER error"); }
	| IDENTIFIER '=' error
		{ ERROR("enumerator: IDENTIFIER = error"); }
	;

declarator
	: pointer direct_declarator
            { if(flag_23 == 1) flag_66 = 1; }
	| direct_declarator
            { if(flag_23 == 1) flag_66 = 1; }
	;

direct_declarator
	: IDENTIFIER
            { if(flag_13 == 1) {
                type_in(yytext);
                flag_16 = flag_13 = 0;
                }
              add_to_list(); }
	| '(' declarator ')'
		{ yyerrok; }
	| direct_declarator '[' ']'
		{ yyerrok; }
	| direct_declarator '[' constant_expression ']'
		{ yyerrok; }
	| direct_declarator '(' parameter_type_list ')'
		{ yyerrok; }
	| direct_declarator '(' ')'
		{ yyerrok; }
	| direct_declarator '(' identifier_list ')'
		{ yyerrok; }
	| '(' error ')'
		{ yyerrok; ERROR("direct_declarator: (error)"); }
	| '(' declarator error
		{ ERROR("direct_declarator: (declarator error"); }
	| direct_declarator '[' error
		{ ERROR("direct_declarator: direct_declarator [error"); }
	| direct_declarator '[' constant_expression error
		{ ERROR("direct_declarator: direct_declarator [constant_expression error"); }
	| direct_declarator '(' error
		{ ERROR("direct_declarator: direct_declarator (error"); }
	| direct_declarator '(' parameter_type_list error
                { ERROR("direct_declarator: direct_declarator (parameter_type_list error"); }
	| direct_declarator '(' identifier_list error
		{ ERROR("direct_declarator: direct_declarator (identifier_list error"); }
	;

pointer
	: '*'
	| '*' type_qualifier_list
	| '*' pointer
	| '*' type_qualifier_list pointer
	;

type_qualifier_list
	: type_qualifier
	| type_qualifier_list type_qualifier
	;

parameter_type_list
	: parameter_list
	| parameter_list ',' ELIPSIS
	;

parameter_list
	: parameter_declaration
	| parameter_list ',' parameter_declaration
	;

parameter_declaration
	: declaration_specifiers declarator
	| declaration_specifiers
	| declaration_specifiers abstract_declarator
	;

identifier_list
	: identifier_list ',' IDENTIFIER { /* PWO */ add_to_list(); }
	| IDENTIFIER { /* PWO */ add_to_list(); }
	;

initializer
	: assignment_expression
	| '{' initializer_list '}'
		{ yyerrok; }
	| '{' initializer_list ',' '}'
		{ yyerrok; }
	;

initializer_list
	: initializer
	| initializer_list ',' initializer
	| initializer_list ',' error
		{ ERROR("initializer_list: initializer_list , error"); }
	;

type_name
	: specifier_qaulifier_list
	| specifier_qaulifier_list abstract_declarator
	;

abstract_declarator
	: pointer
	| direct_abstract_declarator
	| pointer direct_abstract_declarator
	;

direct_abstract_declarator
	: '(' abstract_declarator ')'
		{ yyerrok; }
	| '[' ']'
		{ yyerrok; }
	| direct_abstract_declarator '[' ']'
		{ yyerrok; }
	| '[' constant_expression ']'
		{ yyerrok; }
	| direct_abstract_declarator '[' constant_expression ']'
		{ yyerrok; }
	| '(' ')'
		{ yyerrok; }
	| direct_abstract_declarator '(' ')'
		{ yyerrok; }
	| '(' parameter_type_list ')'
		{ yyerrok; }
	| direct_abstract_declarator '(' parameter_type_list ')'
		{ yyerrok; }
	| direct_abstract_declarator '[' error ']'
		{ ERROR("direct_abstract_declarator: direct_abstract_declarator [ error ]"); }
	| direct_abstract_declarator '(' error ')'
		{ ERROR("direct_abstract_declarator: direct_abstract_declarator ( error )"); }
	;

statement
	: labeled_statement
	| expression_statement
            { in_stmt = 1; }
 	| compound_statement
	| selection_statement
	| iteration_statement
	| jump_statement
	;

labeled_statement
	: IDENTIFIER ':' statement
	| CASE constant_expression ':' statement
	| DEFAULT ':' statement
	| IDENTIFIER ':' error
		{ ERROR("labeled_statement: IDENTIFIER : error"); }
	| CASE error ':' statement
		{ ERROR("labeled_statement: CASE error : statement"); }
	| CASE constant_expression ':' error
		{ ERROR("labeled_statement: CASE constant_expression : error"); }
	| DEFAULT ':' error
		{ ERROR("labeled_statement: DEFAULT : error"); }
	;

expression_statement
	: expression ';'
            { in_stmt = 0; yyerrok; }
	| ';'
            { in_stmt = 0; yyerrok; }
	;

compound_statement
	: start_code declaration_list	{ in_stmt = 1; start_statement(); }
	  statement_list '}'		{ end_code_block(); yyerrok; }
	| start_code declaration_list '}' {end_code_block(); yyerrok; }
	| start_code			{ start_statement(); in_stmt = 1; }
	  statement_list '}'		{ end_code_block(); yyerrok; }
	| start_code '}'		{ end_code_block(); yyerrok; }
	;

start_code
	: '{' {start_code_block();}
	;

statement_list
	: statement_list statement
		{ yyerrok; }
	| statement
	| statement_list error
		{ ERROR("statement_list: statement_list error"); }
	;

selection_statement
	: IF '(' expression ')' {nesting('{');} if_statement {nesting('}');}
	| SWITCH '(' expression ')'
		{ yyerrok; nesting('{'); }
	  statement {nesting('}');}
	| IF error statement
		{ ERROR("selection_statement: IF error statement"); }
	| IF error statement ELSE statement
		{ ERROR("selection_statement: IF error statement ELSE statement"); }
	| SWITCH '(' error ')'
		{ yyerrok; }
	  statement
		{ ERROR("selection_statement: SWITCH (error) statement"); }
	| SWITCH error statement
		{ ERROR("selection_statement: SWITCH error statement"); }
	;

if_statement
	: statement
	| statement ELSE statement
	;


iteration_statement
	: WHILE '(' expression ')' {nesting('{');} statement {nesting('}');}
	| DO {nesting('{');} statement WHILE '(' expression ')' ';' {nesting('}'); yyerrok; }
	| FOR '(' ';' ';' ')'  { in_exp = 0; nesting('{'); } statement {nesting('}');}
	| FOR '(' expression ';' ';' ')' {nesting('{');} statement {nesting('}');}
	| FOR '(' ';' expression ';' ')' {nesting('{');} statement {nesting('}');}
	| FOR '(' ';' ';' expression ')' {nesting('{');} statement {nesting('}');}
	| FOR '(' expression ';' expression ';' ')'  {nesting('{');} statement {nesting('}');}
	| FOR '(' ';' expression ';' expression ')' {nesting('{');} statement {nesting('}');}
	| FOR '(' expression ';' ';' expression ')' {nesting('{');} statement {nesting('}');}
	| FOR '(' expression ';' expression ';' expression ')' {nesting('{');} statement {nesting('}');}
	| WHILE error statement
		{ ERROR("iteration_statement: WHILE error statement"); }
	| DO error
		{ ERROR("iteration_statement: DO error "); }
		WHILE error ';'
		{ yyerrok; ERROR("iteration_statement: DO statement WHILE error ;"); }
	| FOR error statement
		{ ERROR("iteration_statement: FOR error statement"); }
	;

jump_statement
	: GOTO IDENTIFIER ';'
		{ yyerrok; }
	| CONTINUE ';'
		{ yyerrok; }
	| BREAK ';'
		{ yyerrok; }
	| RETURN ';'
		{ yyerrok; }
	| RETURN expression ';'
		{ yyerrok; }
	| GOTO error
		{ ERROR("jump_statement: GOTO error"); }
	| CONTINUE error
		{ ERROR("jump_statement: CONTINUE error"); }
	| BREAK error
		{ ERROR("jump_statement: BREAK error"); }
	| RETURN error
		{ ERROR("jump_statement: RETURN error ;"); }
	;

expression
	: expression ',' assignment_expression
	| assignment_expression
	| expression ',' error
		{ ERROR("expression: expression , error"); }
	;

assignment_expression
	: conditional_expression
	| unary_expression assignment_operator assignment_expression
	| unary_expression assignment_operator error
		{ ERROR("assignment_expression: unary_expression assignment_operator error"); }
	;

assignment_operator
	: ASSIGN_OP
	| '=' { initialize_id(); }
	;

conditional_expression
	: logical_or_expression
	| logical_or_expression '?' expression ':' conditional_expression
	| logical_or_expression '?' error ':' conditional_expression
		{ ERROR("conditional_expression: logical_or_expression '?' error ':' conditional_expression"); }
	| logical_or_expression '?' expression ':' error
		{ ERROR("conditional_expression: logical_or_expression '?' expression ':' error"); }
	;

constant_expression
	: conditional_expression
	;

logical_or_expression
	: logical_and_expression
	| logical_or_expression OR logical_and_expression
	| logical_or_expression OR error
		{ ERROR("logical_or_expression: logical_or_expression OR error"); }
	;

logical_and_expression
	: inclusive_or_expression
	| logical_and_expression AND inclusive_or_expression
	| logical_and_expression AND error
		{ ERROR("logical_and_expression: logical_and_expression AND error"); }
	;

inclusive_or_expression
	: exclusive_or_expression
	| inclusive_or_expression '|' exclusive_or_expression
	| inclusive_or_expression '|' error
		{ ERROR("inclusive_or_expression: inclusive_or_expression | error"); }
	;

exclusive_or_expression
	: and_expression
	| exclusive_or_expression '^' and_expression
	| exclusive_or_expression '^' error
		{ ERROR("exclusive_or_expression: exclusive_or_expression ^ error"); }
	;

and_expression
	: equality_expression
	| and_expression '&' equality_expression
	| and_expression '&' error
		{ ERROR("and_expression: and_expression & error"); }
	;

equality_expression
	: relational_expression
	| equality_expression EQUALITY_OP relational_expression
	| equality_expression EQUALITY_OP error
		{ ERROR("equality_expression: equality_expression EQUALITY_OP error"); }
	;

relational_expression 
	: shift_expression
	| relational_expression RELATIONAL_OP shift_expression
	| relational_expression RELATIONAL_OP error
		{ ERROR("relational_expression: relational_expression RELATIONAL_OP error"); }
	;

shift_expression
	: additive_expression
	| shift_expression SHIFT_OP additive_expression
	| shift_expression SHIFT_OP error
		{ ERROR("shift_expression: shift_expression SHIFT_OP error"); }
	;

additive_expression
	: multiplicative_expression
	| additive_expression '+' multiplicative_expression
	| additive_expression '-' multiplicative_expression
	| additive_expression '+' error
		{ ERROR("additive_expression: additive_expression + error"); }
	| additive_expression '-' error
		{ ERROR("additive_expression: additive_expression - error"); }
	;

multiplicative_expression	
	: cast_expression
	| multiplicative_expression '*' cast_expression
	| multiplicative_expression '/' cast_expression
	| multiplicative_expression '%' cast_expression
	| multiplicative_expression '*' error
		{ ERROR("multiplicative_expression: multiplicative_expression * error"); }
	| multiplicative_expression '/' error
		{ ERROR("multiplicative_expression: multiplicative_expression / error"); }
	| multiplicative_expression '%' error
		{ ERROR("multiplicative_expression: multiplicative_expression % error"); }
	;

cast_expression	
	: unary_expression
	| '(' type_name ')'
		{ yyerrok; }
	  cast_expression
	| '(' error ')' cast_expression
		{ ERROR("cast_expression: ( error ) cast_expression"); }
	;

unary_expression
	: postfix_expression
	| INCREMENT unary_expression
	| DECREMENT unary_expression
	| unary_operator cast_expression
	| SIZEOF unary_expression
	| SIZEOF '(' type_name ')'
		{ yyerrok; }
	| INCREMENT error
		{ ERROR("unary_expression: INCREMENT error"); }
	| DECREMENT error
		{ ERROR("unary_expression: DECREMENT error"); }
	| unary_operator error
		{ ERROR("unary_expression: unary_operator error"); }
	| SIZEOF error
		{ ERROR("unary_expression: SIZEOF error"); }
	| SIZEOF '(' error ')'
		{ ERROR("unary_expression: SIZEOF (error)"); }
	;

unary_operator
	: '&'
	| '*'
	| '+'
	| '-'
	| '~'
	| '!'
	;

postfix_expression
	: primary_expression
	| postfix_expression '[' expression ']'
		{ yyerrok; }
	| postfix_expression '(' argument_expression_list ')'
		{ yyerrok; }
	| postfix_expression '(' ')'
		{ yyerrok; }
	| postfix_expression '.' IDENTIFIER 
		{ /* PWO */ add_to_list(); yyerrok; }
	| postfix_expression INDIRECT_SELECTION IDENTIFIER
		{ /* PWO */ add_to_list(); yyerrok; }
	| postfix_expression INCREMENT
		{ yyerrok; }
	| postfix_expression DECREMENT
		{ yyerrok; }
	| postfix_expression '[' error
		{ ERROR("postfix_expression: postfix_expression [error"); }
	| postfix_expression '[' expression error
		{ ERROR("postfix_expression: postfix_expression [expression error"); }
	| postfix_expression '(' error
		{ ERROR("postfix_expression: postfix_expression (error"); }
	| postfix_expression '(' argument_expression_list error
		{ ERROR("postfix_expression: postfix_expression (argument_expression_list error"); }
	| postfix_expression '.' error
		{ ERROR("postfix_expression: postfix_expression.error"); }
	| postfix_expression INDIRECT_SELECTION error
		{ ERROR("postfix_expression: postfix_expression->error"); }
	;

primary_expression
	: IDENTIFIER { add_to_list(); }
	| CONSTANT
	| string_conat
	| '(' expression ')'
		{ yyerrok; }
	;

string_conat
	: STRING
	| string_conat STRING
	;

argument_expression_list	
	: assignment_expression
	| argument_expression_list ',' assignment_expression
	;

%%

#include "flag.h"
#include "lex.yy.c"

/*     main

       input     : stdin.
       output    : stdout.
       calls     : set_up_modules, yyparse, end_of_parse.

       This procedure starts the parsing process.
*/

main()
{
  set_up_modules();
  yyparse();
  end_of_parse();
  
  exit(0);
}
