/*$Header: calc.c,v 4.1 94/02/02 16:40:58 tedd Exp $*/

/**********************************************************************/
/*                                                                    */
/*          Global File for the MAS Maintainability Tool              */
/*          CS 481 Senior Group Project - Fall 1991                   */
/*          Group Members : Dustin Day, Pat Felzien, Dennis Sims,     */
/*                          Mark Vieselmeyer, Roger Wynn              */
/*          Date File Created : November 5, 1991                      */
/*          Last File Update  : December 7, 1991                      */
/*                                                                    */
/**********************************************************************/

/**********************************************************************/
/*  This file is entitled inputf.c.  It is included into the main.c   */
/*       file.  It handles the metric table (input) and calculates    */
/*       the overall and module percent fits.                         */
/**********************************************************************/

#include <stdio.h>

#include "globals.h"

/********************************************************/
/*  read_met_table:  reads file of metric bounds        */
/*  CS-481  University of Idaho,  Fall 1991  (Felzien)  */
/********************************************************/
read_met_table(mod_table, mod_tab_size)
struct mod_result *mod_table[];
int mod_tab_size;
{
  FILE *mf;
  int i;
  char lable[30];
  char hold[80];	/*  buffer to hold header of metrics table */
  int flag = 0;
 

  if ((mf = fopen (HOME"mas.met_table", "r")) == NULL) 
  {
    fprintf(stderr,
            "<ERROR> Aborting! Metric bound table not found.\n");
    fprintf(stderr,"Check Makefile instructions\n");
    exit(1);
  }

  /*  check for blank lines and header  */
  fgets (hold, 80, mf);
  while (flag == 0) 
  {

    if (strcmp (hold, "\n") == 0) 
      fgets (hold, 80, mf);
    else flag = 1;

  }

  /*  read in overall bounds  */
  for (i = 0; i < 19; i++) 
  {  
    fscanf(mf, "%s %d %d %d\n", lable, &met_bounds[0][i], 
           &met_bounds[1][i], &met_bounds[2][i]);
  }

  /*  check for blank lines and header  */
  flag = 0;
  fgets (hold, 80, mf);
  while (flag == 0) 
  {
    if (strcmp (hold, "\n") == 0) 
      fgets (hold, 80, mf);
    else flag = 1;
  }

  /*  read in module bounds  */
  for (i = 19; i < 37; i++) 
  {
    fscanf (mf, "%s %d %d %d\n", lable, &met_bounds[0][i], 
            &met_bounds[1][i], &met_bounds[2][i]);
  }
  fclose(mf);
}

/********************************************************/
/*  calc_per_dev:  calculates the percent deviation     */
/*  CS-481  University of Idaho,  Fall 1991  (Felzien)  */
/********************************************************/

float calc_per_dev(x, i, j, mod_table)
float x;
int i;
int j;
struct mod_result *mod_table[];
{
  float per_dev = 0.00;
  float weighted_dev = 0.00;

  /*  check if no penalty  */
  if (x >=  (float)met_bounds[0][i] && x <= (float)met_bounds[1][i])
    per_dev = 0.0;

  /*  else, calculate penalty  */
  else 
  {
    if (x > (float)met_bounds[1][i])
      per_dev = (x - (float)met_bounds[1][i])/(float)met_bounds[1][i];
    else 
      per_dev = ((float)met_bounds[0][i] - x)/(float)met_bounds[1][i];

    if (j >= 0 && met_bounds[2][i] > 0)
    {  
      mod_table[j]->num_except++;
    }
  }
  if (per_dev > 1.0) per_dev = 1.0;

  weighted_dev = per_dev * (float)met_bounds[2][i];

  return(weighted_dev);
}


/*******************************************************************/
/*  calc_overall_fit:  calculates overall percent maintainability  */
/*  CS-481  University of Idaho,  Fall 1991  (Felzien)             */
/*******************************************************************/

void calc_overall_fit(mod_table, mod_tab_size)
struct mod_result *mod_table[];
int mod_tab_size;
{
  int i = 0;
  float con_dev = 0.0;
  float data_dev = 0.0;
  float typo_dev = 0.0;
  float x = 0.0;
  float w1 = 0.0;
  float w2 = 0.0;
  float w3 = 0.0;
  float ave_fit = 0.0;
  float sum_percent = 0.0;

  /*  control metrics  */

  x = all_metrics.ave_LOC;
  con_dev += calc_per_dev(x,i,-1, mod_table);
  i++;
  x = all_metrics.ave_vsubg;
  con_dev += calc_per_dev(x,i,-1,mod_table );
  i++;
  x = all_metrics.ave_Halv;
  con_dev += calc_per_dev(x,i,-1,mod_table );
  i++;
  x = all_metrics.ave_Hale;
  con_dev += calc_per_dev(x,i,-1,mod_table );
  i++;
  x = all_metrics.ave_fanin;
  con_dev += calc_per_dev(x,i,-1,mod_table );
  i++;
  x = all_metrics.ave_nesting;
  con_dev += calc_per_dev(x,i,-1, mod_table);
  i++;

  /*  data metrics  */

  x = all_metrics.ave_num_iocalls;
  data_dev += calc_per_dev(x,i,-1, mod_table);
  i++;
  x = all_metrics.percent_welldecl;
  data_dev += calc_per_dev(x,i,-1, mod_table);
  i++;
  x = all_metrics.ave_var_span;
  data_dev += calc_per_dev(x,i,-1, mod_table);
  i++;
  x = all_metrics.ave_num_params;
  data_dev += calc_per_dev(x,i,-1, mod_table);
  i++;
  x = all_metrics.percent_tot_init_vars;
  data_dev += calc_per_dev(x,i,-1, mod_table);
  i++;

  /*  layout metrics  */

  x = all_metrics.percent_mod_with_header;
  typo_dev += calc_per_dev(x,i,-1, mod_table);
  i++;
  x = all_metrics.percent_mods_with_comments;
  typo_dev += calc_per_dev(x,i,-1, mod_table);
  i++;
  x = all_metrics.percent_mods_with_vert_sp;
  typo_dev += calc_per_dev(x,i,-1, mod_table);
  i++;
  x = all_metrics.percent_mods_with_horiz_sp;
  typo_dev += calc_per_dev(x,i,-1, mod_table);
  i++;
  x = all_metrics.percent_mods_with_indent;
  typo_dev += calc_per_dev(x,i,-1, mod_table);
  i++;
  x = all_metrics.ave_per_uncrowd_stmnt;
  typo_dev += calc_per_dev(x,i,-1, mod_table);
  i++;
  x = all_metrics.ave_percent_comments;
  typo_dev += calc_per_dev(x,i,-1, mod_table);
  i++;
  x = all_metrics.ave_per_mean_vars;
  typo_dev += calc_per_dev(x,i,-1, mod_table);

  /*  add up weights  */

  for (i = 0; i < 6; i++) 
  {
    w1 += met_bounds[2][i];
  }
  
  for (i = 4; i < 9; i++) 
  {
    w2 += met_bounds[2][i];
  }
  
  for (i = 7; i < 15; i++) 
  {
    w3 += met_bounds[2][i];
  }

  /* Lowther92: calculate control_fit, Data_fit and Typo_fit.*/
  all_metrics.percent_tot_control_fit = (1.0 - con_dev / w1) * 100;
  all_metrics.percent_tot_data_fit    = (1.0 - data_dev / w2) * 100;
  all_metrics.percent_tot_typo_fit    = (1.0 - typo_dev / w3) * 100;
   
  /*  calculate overall fit  */

  /* TED '94--only using the code below now. 
   */
    ave_fit =  100 * (all_metrics.percent_tot_control_fit/100)
             *  (all_metrics.percent_tot_data_fit/100)
             *  (all_metrics.percent_tot_typo_fit/100);

  /* TED '94--Not using this anymore
   *    ave_fit = (all_metrics.percent_tot_control_fit +
   *      all_metrics.percent_tot_data_fit +
   *      all_metrics.percent_tot_typo_fit) /3;
   */

  /*Lowther93: assign it to the all metrics structure. */
  all_metrics.percent_overall_fit = ave_fit;
}


/******************************************************************/
/*  calc_mod_fit:  calculates the module percent maintainability  */
/*  CS-481  University of Idaho, Fall 1991  (Felzien)             */
/******************************************************************/

void calc_mod_fit(mod_tab_size, mod_table)
int mod_tab_size;
struct mod_result *mod_table[];
{
  int i = 19;
  float con_dev = 0.0;
  float data_dev = 0.0;
  float typo_dev = 0.0;
  float x = 0.0;
  int j;
  float w1 = 0.0;
  float w2 = 0.0;
  float w3 = 0.0;
  float percent_fit = 0.0;

  /*  add up weights  */
  for (j = 19; j < 25; j++) 
  {
    w1 += (float)met_bounds[2][j];
  }
  
  for (j = 25; j < 30; j++) 
  {
    w2 += (float)met_bounds[2][j];
  }
  
  for (j = 30; j < 37; j++) 
  {
    w3 += (float)met_bounds[2][j];
  }

  /*  for each module do this:  */
  for (j = 0; j < mod_tab_size; j++) 
  {
    i = 19;
    con_dev = 0;
    data_dev = 0;
    typo_dev = 0;

    /*  control metrics  (also calculates exceptions)  */

    x = mod_table[j]->LOC;
    con_dev += calc_per_dev(x,i,j, mod_table);
    i++;
    x = mod_table[j]->vsubg;
    con_dev += calc_per_dev(x,i,j, mod_table);
    i++; 
    x = mod_table[j]->max_nest;
    con_dev += calc_per_dev(x,i,j, mod_table);
    i++;
    x = mod_table[j]->fanin;
    con_dev += calc_per_dev(x,i,j, mod_table);
    i++;
    x = mod_table[j]->Halv;
    con_dev += calc_per_dev(x,i,j, mod_table);
    i++;
    x = mod_table[j]->Hale;
    con_dev += calc_per_dev(x,i,j, mod_table);
    i++;
    
    
    /*  data metrics  */

    x = mod_table[j]->num_iocall;
    data_dev += calc_per_dev(x,i,j, mod_table);
    i++;
    x = mod_table[j]->ave_welldecl;
    data_dev += calc_per_dev(x,i,j, mod_table);
    i++;
    x = mod_table[j]->ave_span;
    data_dev += calc_per_dev(x,i,j, mod_table);
    i++;
    x = mod_table[j]->num_params;
    data_dev += calc_per_dev(x,i,j, mod_table);
    i++;
    x = mod_table[j]->percent_initvars;
    data_dev += calc_per_dev(x,i,j, mod_table);
    i++;
    
    /*  layout metrics  */

    x = mod_table[j]->percent_uncrowd_stmnts;
    typo_dev += calc_per_dev(x,i,j, mod_table);
    i++;
    x = mod_table[j]->num_mean_vars; /** check this - they had var_mean ***/
    typo_dev += calc_per_dev(x,i,j, mod_table);
    i++;
    x = mod_table[j]->percent_comments;
    typo_dev += calc_per_dev(x,i,j, mod_table);
    i++;
    
    if (mod_table[j]->header == 0) 
    {
      typo_dev += (1 * met_bounds[2][i]);
      if (met_bounds[2][i] > 0)
        mod_table[j]->num_except++;
    }
    i++;

    if (mod_table[j]->horiz_sp == 0) 
    {
      typo_dev += (1 * met_bounds[2][i]);
      if (met_bounds[2][i] > 0)
        mod_table[j]->num_except++;
    }
    i++;
    
    if (mod_table[j]->vert_sp == 0) 
    {
      typo_dev += (1 * met_bounds[2][i]);
      if (met_bounds[2][i] > 0)
        mod_table[j]->num_except++;
    }
    i++;

    if (mod_table[j]->indent == 0) 
    {
      typo_dev += (1 * met_bounds[2][i]);
      if (met_bounds[2][i] > 0)
        mod_table[j]->num_except++;
    }

    /*  Lowther92: Store off the con, data, and typo values for reporting...
     *  similar to what was done for the overall fit.
     */

    mod_table[j]->percent_control_fit = (1.0 - con_dev / w1) * 100;
    mod_table[j]->percent_data_fit = (1.0 - data_dev / w2) * 100;
    mod_table[j]->percent_typo_fit = (1.0 - typo_dev / w3) * 100;

    /* Lowther92:*/

    /*  calculate metric fit  */
    
    /* TED '94--Only use this code now.
     */
      percent_fit =  100 * (mod_table[j]->percent_control_fit/100)
                  * (mod_table[j]->percent_data_fit/100)
                  * (mod_table[j]->percent_typo_fit/100);


    /* TED '94--not using the code below anymore
     *      percent_fit = (mod_table[j]->percent_control_fit
     *         + mod_table[j]->percent_data_fit
     *         + mod_table[j]->percent_typo_fit) /3;
     */ 

    mod_table[j]->percent_fit = percent_fit;
    
  }
}

void ave_percent_uncrowd (mod_tab_size, all_metrics)
int mod_tab_size;
struct all_result *all_metrics;

/********************************************************************/
/*  This function calculates the average percentage of uncrowded    */
/*  statements from all modules. 			            */
/*                                                                  */
/*  Written by:     Dennis Sims      10/14/91                       */
/*  Last Modified:                   11/16/92 Theresa Rhoades       */
/********************************************************************/
{
  int i;
  float sum_percent = 0;      /*  sum percentages over modules */
  float avg_per_uncrowd = 0;  /*  calculates ave % of uncrowded statements */

	/*  calculate the average percentage of uncrowded statements
	    by summing the percentages of uncrowded statements for
	    each module and dividing by the number of modules */

   for (i= 0;i < mod_tab_size;i++)
     sum_percent += mod_table[i]->percent_uncrowd_stmnts;

   if (num_mods <= 0)
     all_metrics->ave_per_uncrowd_stmnt = 0.00;
   else
   {
     avg_per_uncrowd = ((float) sum_percent / (float)num_mods);
     all_metrics->ave_per_uncrowd_stmnt = avg_per_uncrowd;
   }
}

void calc_exceptions(mod_tab_size,all_metrics,mod_table)
int mod_tab_size;
struct all_result all_metrics;
struct mod_result *mod_table[];
/****************************************************************************/
/*                                                                          */
/*         MODULE NAME       : calc_exceptions                              */
/*         CREATION DATE     : September 1, 1991                            */
/*         LAST UPDATE       : November 16, 1992                            */
/*         PROGRAMMER        : Roger Wynn                                   */
/*         UPDATE PROGRAMMER : Theresa Rhoades                              */
/*                                                                          */
/****************************************************************************/

{                               /* calc_exceptions begin */
  int more;                     /* loop index */
  int except_per_mod = 0;	/* count exceptions per mod */
  int mods_with_except = 0;	/* count number of modules with exceptions */

  for (more = 0;more <  mod_tab_size;more++)  /* for all the modules ... */
    if (mod_table[more] -> num_except > 0)    /* if there are exceptions ... */
    {
      mods_with_except++;    
      except_per_mod += mod_table[more] -> num_except;
    }

  if (all_metrics.tot_mods == 0)
  {
    all_metrics.ave_except_per_mod = 0.00; 
    all_metrics.percent_mods_except = 0.00;
  }
  else
  {
    all_metrics.ave_except_per_mod =  (float)(except_per_mod
                                              /(float)all_metrics.tot_mods);
    all_metrics.percent_mods_except = (float)((mods_with_except
                                               /(float) all_metrics.tot_mods)
                                              * 100);
  }
  percent_mods_with_except = all_metrics.percent_mods_except;
  ave_exceptions = all_metrics.ave_except_per_mod;
  
}   /* calc_exceptions end */

calc_fit(all_metrics, mod_table, mod_tab_size)
struct all_result all_metrics;
struct mod_result *mod_table[];
int mod_tab_size;
/**************************************************************************/
/*                                                                        */
/*     MODULE NAME       : calc_fit  (previously called "met_table_drive" */
/*     CREATION DATE     : September 1, 1991                              */
/*     LAST UPDATE       : November 16, 1992                              */
/*     PROGRAMMER        : Pat Pelzien                                    */
/*     UPDATE PROGRAMMER : Theresa Rhoades                                */
/*                                                                        */
/**************************************************************************/


/* The purpose of this module is to drive the code that reads in the table of
 acceptable metric ranges and values and then calculates the percentage of
 overall fit and the percentage of module fit for each module.

 This module gets called by  calculate_metrics()
*/

{ /* met_table_drive begin */
  /* read in acceptable metric ranges */
  read_met_table(mod_table, mod_tab_size); 
  
  /* calculate percentage of module fit */
  calc_mod_fit(mod_tab_size, mod_table);

  /*  calculate percentage of overall fit */ 
  calc_overall_fit(mod_table, mod_tab_size);
} /* met_table_drive end */


void calculate_metrics()
{
  fprintf(stdout, " ... computing ");
  ave_percent_uncrowd(mod_tab_size, &all_metrics);  
  calc_fit(all_metrics, mod_table, mod_tab_size);
  calc_exceptions(mod_tab_size, all_metrics, mod_table);
}
