/*$Header: calls.c,v 4.0 94/02/01 20:20:09 tedd Exp $*/

#include "sp-code.h"
#include "common.h"

/*    is_external_call

      called by : check_procedures

      This procedure checks to see if the procedure call is call to open
      up a stream.  This is used to check for external files.
*/

int is_external_call(char *string)
{
  struct link_list *temp;
  char *operators[EXTERNAL_CALL_SIZE] = 
    {"creat","fopen","freopen","open","tmpfile"};

  int low = 0, high = EXTERNAL_CALL_SIZE, mid =EXTERNAL_CALL_SIZE/2;
  int equal = 0;

  for(equal = strcmp(string, operators[mid]);
              equal && high-low>1;
              equal = strcmp(string, operators[mid]))
  {
    if(equal > 0)
    {
      low = mid;
      mid = (high+low)/2;
    }
  else if(equal < 0)
    {
      high = mid;
      mid = (high+low)/2;
    }
  }
 
  if(equal == 1)
  {
    mid = 0;
    equal = strcmp(string, operators[mid]);
  }
	
  if(equal == 0)
    return 1;
  else
    return 0;
}


/*    is_user_defined

      called by : check_procedures

      This procedure checks the current procedure list to see if the
      called procedure is predefined.
*/

int is_user_defined(char *string)
{
  struct module_info *temp = GLOBAL->code_block;
  while( temp != NULL )
  {
    if(!strcmp(string, temp->module_name))
    {
      CURRENT_MODULE->fan_out++;
      return 1;
    }
    temp = temp -> code_block;
  }
  return 0;
}


/*    is_recursive

      called by : check_procedures

      This procedure checks the called procedure name with the current
      procedures module name.
*/

int is_recursive(char *string)
{
  int recurse;
  struct module_info *temp = CURRENT_MODULE;
  while(temp->code_block != GLOBAL)
    temp = temp->code_block;
  recurse = strcmp(string, temp->module_name);
  if(recurse == 0)
  {
    temp->recursive_calls++;
    return 1;
  }
  return 0;
}


/*    is_io_call

      called by : check_procedures

      This procedure checks to see if the procedure name is a
      input/output call.
*/

int is_io_call(char *string)
{
  char *operators[IO_CALL_SIZE] = 
      {"close","fclose","feof","ferror","fflush","fgetc","fgetpos",
       "fgets","fpos_t","fprintf","fputc","fputs","fread","fscanf",
       "fseek","fsetpos","ftell","fwrite","getc","getchar","gets",
       "getw","lseek","pclose","perror","printf","putc","putchar",
       "puts","read","rename","scanf","sprintf","sscanf","ungetc",
       "unlike","vfprintf","vprintf","vsprintf","write"};

  int low = 0, high = IO_CALL_SIZE, mid = IO_CALL_SIZE/2;
  int equal = 0;

  for (equal = strcmp( string, operators[mid]);
       equal && high-low>1;
       equal = strcmp(string, operators[mid]))
  {
    if (equal > 0)
    {
      low = mid;
      mid = (high+low)/2;
    }
    else if(equal < 0)
    {
      high = mid;
      mid = (high+low)/2;
    }
  }
	
  if (equal == 1)
  {
    mid = 0;
    equal = strcmp(string, operators[mid]);
  }
	
  if (equal == 0)
  {
    CURRENT_MODULE->num_of_io_calls++;
    return 1;
  }
  return 0;
}


/*     check_procedures

       called by :
       calls : is_recursive, is_user_defined, is_io_call.

       This procedure checks the called procedure name to see what type
       of procedure call it is.
*/

void check_procedures(char *string)
{
  if (is_recursive(string))
    return;
  if (is_user_defined(string))
    return;
  if (is_io_call(string))
    return;
  CURRENT_MODULE->fan_out++;
}












