/*$Header: globals.h,v 4.0 94/02/01 20:27:40 tedd Exp $*/

#define TRUE  1
#define FALSE 0
#define MOD_ARRAY 2000	  /*  Used to build array of Module Metrics */

/*lowther93: default name for both the .report and .metrics file */
#define DEFAULT_NAME "result"

/* module metrics */

/* int more; */
/* int index; */

extern int mod_tab_size ;    /*  size of module table; equals num_mods */
extern int num_mods ;	     /*  number of modules processed */
extern int met_bounds[][37]; /*  two-dimensional array holding metrics table */

float percent_mods_with_except;
float ave_exceptions; 
int total_welldecl;

struct mod_result 
{
  char name[25];		/*  name of module */
  int num_except;		/*  number of exceptions */
  float percent_fit;	        /*  percentage fit for module */
  int num_stmnts;		/*  number of statements in module */

/* Lowther92: Added next three to get the intermediate percent fit for each
              module.
*/
  float percent_control_fit;
  float percent_data_fit;
  float percent_typo_fit;
  /* Lowther92:*/

  /* CONTROL :  Control Metric Variables for Module Metrics */
  
  /*  TED '94--added NCSS to structure.
   */
  int NCSS;             /*  Non Commented Source Statements */
  int LOC;		/*  Lines of Code in Module */
  int vsubg;		/*  V(g) for module */
  float Halv;		/*  Halstead's volume */
  float Hale;		/*  Halstead's effort */
  int max_nest;	/*  Maximum nesting in module */
  int num_rec_calls;	/*  Number of recursive calls in module */
  int fanin;		/*  Fan in for module */
  int fanout;		/*  Fan out for module */
  
  /* DATA :  Data Metric Variables for Module Metrics */
  
  int num_iocall;	/*  Number of IO calls in module */
  int ext_files;	/*  Number of external files used */
  int unusedvar;	/*  Number of unused variables in module */
  int global;		/*  Number of global variables in module */
  int num_local;	/*  number of local variables in a module */
  int num_params;	/*  Number of formal parameters/variables */
  int num_welldecl;	/*  Number of well declared variables */
  float ave_welldecl;   /*   average number of well-declared variables */
  float percent_welldecl;	/*  percentage well declared variables */
  float percent_meaningful;	/*  percentage meaningful variables */
  int num_initvars;		/*  number initialized variables */
  float percent_initvars;	/*  percentage initialized variables */
  int max_span;	                /*  Maximum span over module */
  float ave_span;	        /*  Average variable span of module */
  
  /* TYPOGRAPHIC :  Typeographic Metric Variables for Module Metrics */

  float percent_comments;	/*  percentage of comments */
  int num_comments;		/*  number of comments */
  int header;			/*  Boolean; TRUE - header exists 
                                             FALSE does not exist */
  int horiz_sp;  		/*  Horizontal spacing in module */
  int vert_sp;			/*  Vertical spacing in module */
  int indent;			/*  Indentation */
  int num_uncrowd_stmnt;
  float percent_uncrowd_stmnts;	/*  Percentage of uncrowded statements */
  int num_mean_vars;		/*  Number of meaningful variables */
};


extern struct mod_result *mod_table[];


/*****************************************************************************
* Overall Program Metrics :  The following section contains the data
*	structures used to hold overall program metrics.
*
*	struct all_result:  	Data Structure holding all overall program
*				metrics.
*****************************************************************************/

struct all_result {
  float mods_with_except;	/*  Number of modules with exceptions */
  float ave_except_per_mod;	/*  Average exceptions per module */
  float percent_mods_except;    /*  percentage of modules with exceptions */
  float percent_overall_fit;	/*  Average total fit of modules */

/* Lowther92: Added next three to save total values of fit for control,
              data, and typo.
*/
  float percent_tot_control_fit;
  float percent_tot_data_fit;
  float percent_tot_typo_fit;
  /* Lowther92:*/
  
  /*CONTROL :  Control Metric Variables for Overall Metrics*/

  int max_vsubg;	/*  Maximum V(g) in a module */
  int max_nesting;	/*  Maximum nesting in program */
  /*  TED '94--Added max_NCSS to structure 
   */
  int max_NCSS;	        /*  Maximum Non Commented Source Statements */
  int max_LOC;		/*  Maximum number of lines of code in a module */
  int max_fanin;	/*  Maximum fan-in for a module */
  int max_fanout;	/*  Maximum fan-out for a module */
  int max_rec_call;	/*  Maximum number of recursive calls in a module */
  int tot_rec_mod;	/*  Total recursive modules present */
  int tot_mods;	        /*  Total number of modules in program */
  /*  TED '94--Added tot_NCSS to structure 
   */
  int tot_NCSS;   	/*  total Non Commented Source Statements */
  int tot_LOC;		/*  Total number of lines of code in the program */
  int tot_vsubg;	/*  Total V(g) for the program */
  float ave_vsubg;	/*  Average V(g) per module */
  float tot_Halv;	/*  total Halstead's volume */
  float ave_Halv;	/*  average Halstead's volume */
  float max_Halv;	/*  maximum Halstead's volume */
  float tot_Hale;	/*  total Halstead's effort */
  float ave_Hale;	/*  average Halstead's effort */
  float max_Hale;	/*  maximum Halstead's effort */
  float ave_fanin;	/*  Average fan-in in program */
  float ave_nesting;	/*  Average nesting in program */
  /*  TED '94--Added ave_NCSS to structure 
   */
  float ave_NCSS;  	/*  Average Non Commented Source Statements */
  float ave_LOC;	/*  Average lines of code per module */
  
  /* DATA :  Data Metric Variables for Overall Program Metrics */
  
  int num_prep;         /*  Number of SOMETHING */
  int tot_local;	/*  Total Local Variables */
  int tot_vars;	        /*  Total number of variables */
  int tot_global;	/*  Total global variables */
  int tot_unusedvar;	/*  Total number of unused variables */
  float ave_num_params; /*  average number of parameters per module */
  float ave_init_vars;  /*  average initialized variables */
  float percent_tot_init_vars;	/*  percentage initialized variables */
  int max_var_mod;	        /*  maximum variables in a module */
  float ave_var_mod;	   /*  average variables per module */
  int tot_ext_files;       /*  Total number of external files accessed */
  float ave_num_iocalls;   /*  Average number of IO calls per module */
  int tot_welldecl;	   /*  total well declared variables */
  float ave_welldecl;	   /*  Average number of well declaired variables */
  float percent_welldecl;  /*  percentage of well declared variables */
  int max_var_span;	   /*  maximum variable span in a module */
  float ave_var_span;	   /*  Average variable span per module */

  /*TYPOGRAPHIC :  Typographics Variables for Overall Program Metrics */

  int tot_uncrowd_stmnt;	/*  total uncrowded statements */
  float ave_uncrowd_stmnt;	/*  Average uncrowded statements per module */
  float ave_per_uncrowd_stmnt;  /*  Average percent uncrowded statements */
  int mod_with_indent;	        /*  Number of modules with indentions */
  int tot_comments;             /*  total number of comments */
  float ave_comments_per_mod;   /*  average comments per module */
  float ave_percent_comments;	/*  average percent of comments per module */
  int mod_with_header;	        /*  Number of modules with headers */

  /* Lowther92: added percent_mod_with_header*/
  float percent_mod_with_header;
  /* Lowther92:*/

  int mod_with_comments;	/*  Number of modules with comments */
  int mod_with_vert_sp;	        /*  Number of modules with vertical spacing */
  int mod_with_horiz_sp;	/*  Number of modules with horizontall spacing */
  float percent_mods_with_horiz_sp; /*  percent modules with horizonal spacing */
  float percent_mods_with_vert_sp; /*  percent modules with vertical spacing */
  float percent_mods_with_indent; /*  percent modules with indention  */
  float percent_mods_with_comments;   /*  percent modules with comments */
  int tot_mean_vars;		      /*  Total meaningful vars */
  float ave_per_mean_vars;	      /*  average meaningful variables */
};

extern struct all_result all_metrics;

extern struct mod_list *mod_name[];
extern int stop;

/*Lowther93: this should not be external here
  extern char delimiter; */

char delimiter;

char inputfilename[255]; /*lowther93: original filename 
                           used in the report output. */
char outfilename[255];   /*lowther93: output filename for the .metrics file. */
char output_file[255];   /*lowther93: output filename for the .report file. */

/*lowther93: no longer used. Output always goes to the current directory.
  extern char outdirname[];
*/











