/*$Header: identifiers.c,v 4.0 94/02/01 20:21:36 tedd Exp $*/

#include "sp-code.h"
#include "common.h"

/*   problem's with the identifier tree:
 *	if a variable is addigned to a global variable in it's declaration,
 *	the global variable will be seen as a local variable.
 */

/*    add_list_to_tree

      called by : add_identifier_tree and change_identifier_tree

      This procedure adds the first node's information in the list into
      the newly made node in the identifier tree.
*/

void add_list_to_tree( struct tree *tree )
{
  struct id_info *list_id = start_list->information,
  *tree_id = tree->information;
  
  tree_id->id_name = copy_string( list_id->id_name );
  tree_id->first_line_seen = tree_id->last_line_seen = 
    list_id->first_line_seen;

  if( !tree_id->initialized ) tree_id->initialized = 1;
  tree_id->num_of_times_seen++;
  tree_id->initialized = list_id->initialized;
}

/*     add_identifier_tree

       calls: add_list_to_tree

       This procedure adds the declared identifier to the identifier tree
       in the current module.
*/

void add_identifier_tree()
{
  struct tree *temp_tree = CURRENT_MODULE->identifiers;
  char *string = start_list->information->id_name;
  struct id_info *temp_id;
  int added = 0, diff;
  
  if(temp_tree)
  {
    temp_id = temp_tree->information;
    
    for(diff = strcmp( string, temp_id->id_name);
        diff && !added;
        temp_id = temp_tree->information,
        diff = strcmp(string, temp_id->id_name))
    {
      if(diff<0)
      {
        if(temp_tree->left_node)
          temp_tree = temp_tree->left_node;
        else
        {
          
          temp_tree->left_node=malloc_tree();
          temp_tree = temp_tree->left_node;
          add_list_to_tree(temp_tree);
          added = 1;
        }
      }
      else if(diff > 0)
      {
        if(temp_tree->right_node)
          temp_tree = temp_tree->right_node;
        else
        {
          temp_tree->right_node=malloc_tree();
          temp_tree = temp_tree->right_node;
          add_list_to_tree(temp_tree);
          added = 1;
        }
      }
    }
  }
  else
  {
    temp_tree = CURRENT_MODULE->identifiers = malloc_tree();
    temp_tree->information->id_name = 
      copy_string(start_list->information->id_name);
    temp_tree->information->first_line_seen = 
      temp_tree->information->last_line_seen = 
        start_list->information->first_line_seen;
    temp_tree->information->num_of_times_seen++;
    temp_tree->information->initialized = start_list->information->initialized;

    /* PWO */
    if (DEBUG) printf("add_id_tree: %s",temp_tree->information->id_name);
    if (DEBUG && (CURRENT_MODULE == GLOBAL)) printf(" GLOBAL\n");
    else if (DEBUG) printf("\n"); 
  }
}

/*     change_identifier_tree

       calls : add_list_to_tree

       This procedure changes the information in the first identifier
       tree that it finds it in.  if change_identifier_tree doesn't find
       in anywhere it will add it to the global identifier tree.
*/

void change_identifier_tree()
{
  struct module_info *temp_mod = CURRENT_MODULE;
  struct tree *temp_tree;
  char *string = start_list->information->id_name;
  int added = 0, diff = 1, end = 0;
  
  for( ; temp_mod && diff ; temp_mod = 
      (temp_mod==GLOBAL) ? NULL :temp_mod->code_block)
  {
    end = 0;
    
    if(temp_mod->identifiers)
    {
      temp_tree = temp_mod->identifiers;
      for(diff = strcmp(string, temp_tree->information->id_name);
          diff && !added && !end;
          diff = strcmp(string, temp_tree->information->id_name))
      {
        if(diff<0)
        {
          if(temp_tree->left_node)
            temp_tree = temp_tree->left_node;
          else
          {
            if(temp_mod == GLOBAL)
            {
              temp_tree->left_node=malloc_tree();
              temp_tree = temp_tree->left_node;
              add_list_to_tree(temp_tree);
              added = 1;
              
              /* DEA '93: checks to see if identifier is in a module, if it is 
               *          then adds the identifier to the modules identifier
               *          list. */
               
              if (where_in_code == WHERE_CODE)
                add_identifier_tree();
            }
            else
              end = 1;
          }
        }
        else if(diff > 0)
        {
          if(temp_tree->right_node)
            temp_tree = temp_tree->right_node;
          else
          {
            if(temp_mod == GLOBAL)
            {
              temp_tree->right_node=malloc_tree();
              temp_tree = temp_tree->right_node;
              add_list_to_tree(temp_tree);
              added = 1;

              /* DEA '93: checks to see if identifier is in a module, if it is 
               *          then adds the identifier to the modules identifier
               *          list. */
               
              if (where_in_code == WHERE_CODE)
                add_identifier_tree();
            }
            else
              end = 1;
          }
        }
      }
      if(diff == 0 && added == 0)
      {
        temp_tree->information->num_of_times_seen ++;
        temp_tree->information->last_line_seen =
          start_list->information->first_line_seen;
        if(!temp_tree->information->initialized)
          temp_tree->information->initialized =
            start_list->information->initialized;

        /* DEA '93: checks to see if identifier is in the GLOBAL identifiers
         *          list and in a module, if it is then adds the identifier 
         *          to the modules identifier list. */

        if ((temp_mod == GLOBAL) && (where_in_code == WHERE_CODE))
          add_identifier_tree();

        /* PWO */
        if (DEBUG) printf("chg_id_tree: %s",temp_tree->information->id_name);
        if (DEBUG && (temp_mod == GLOBAL)) printf(" GLOBAL\n");
        else if (DEBUG) printf("\n"); 
      }
    }
    else if( temp_mod == GLOBAL )
    {
      temp_tree = temp_mod->identifiers = malloc_tree();
      add_list_to_tree( temp_tree );
              
      /* DEA '93: checks to see if identifier is in a module, if it is 
       *          then adds the identifier to the modules identifier
       *          list. */
               
      if (where_in_code == WHERE_CODE)
        add_identifier_tree();
    }
  }
  if(!temp_mod)
    CURRENT_MODULE->num_of_globals_used++;
}

/*     traverse_identifier_tree

       called by : end_code_block.

       This procedure traverses the identifier tree gathering the needed
       information.  traverse_identifier_tree will remove the nodes in
       tree as it goes.
 */

void traverse_identifier_tree( struct tree *node )
{
  struct id_info *id_temp;
  int span;
  if(node == NULL)
    return;
  if(node->left_node) traverse_identifier_tree(node->left_node);
  if(node->right_node) traverse_identifier_tree(node->right_node);
  
  id_temp = node ->information;
  CURRENT_MODULE->num_of_identifiers ++;

  if(id_temp->first_line_seen == id_temp->last_line_seen)
    CURRENT_MODULE->num_of_unused_identifiers++;

  CURRENT_MODULE->num_of_initalized_identifiers += id_temp->initialized;

  if( strlen(id_temp->id_name ) > 4)
    CURRENT_MODULE->num_of_meaningful_identifiers ++;
  span = id_temp->last_line_seen - id_temp->first_line_seen;
  CURRENT_MODULE->average_span += span;
  if(span > CURRENT_MODULE->max_span)
    CURRENT_MODULE->max_span = span;
  
  if (DEBUG) printf("identifier - %s - seen %d times with a span of %d\n", 
      node->information->id_name, node->information->num_of_times_seen, span );
  
  CURRENT_MODULE->operands_unique++;
  CURRENT_MODULE->operands_total += node->information->num_of_times_seen;
  
  free_tree(node);
}
