/*$Header: main.c,v 4.0 94/02/01 20:21:55 tedd Exp $*/


/* ****************************************************************************
	FILE:   main.c

                      Main File for the MAS Maintainability Tool
                      CS 481 Senior Group Project - Fall 1991
                      Group Members : Craig Anderson, Kelly Hall, Vince Lowe, 
					Theresa Rhoades, Dayne Steed
                      Date File Created : October 1, 1992
                      Last File Update  : December 1, 1992   KMH
                           File Update  : November  2, 1992

****************************************************************************/


/* This file is the main program file for the MAS system. The file 
   contains the main program modules and all other MAS system files
   are included in it. The MAS system takes a file of compilable ANCII
   C code and extracts a number of metrics from in to determine weather
   or not the code is maintainable. The metrics are compared against a
   table of acceptable ranges for each metric and the MAS system caculates
   a percentage of fit based on the allowed ranges. The program was produced
   as a group senior design project at the University of Idaho in the Fall
   of 1991 as indicated above. For additional information on this file and 
   the MAS system, refer to the MAS Users Manual or the MAS Software
   Support Document.
*/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include "globals.h"

void Intro()
{
  fprintf(stdout,"\n\n\n\tMAS-C Version 4.0 ... parsing ");
}

struct mod_result *module_alloc(size)
int size;

/******************************************************************************

               MODULE NAME       : module_alloc
               CREATION DATE     : September 1, 1991
               LAST UPDATE       : November 20, 1991
               PROGRAMMER        : Roger Wynn
               UPDATE PROGRAMMER : Roger Wynn

******************************************************************************/


/*                              PARAMETER DICTIONARY                         */
/*                                                                           */
/*     1) size : The size of the data structure to be created. It is used    */
/*               in the call to the library function "malloc" in creating    */
/*               a new node in the module table.                             */

/*                              GLOBAL DICTIONARY                            */
/*                                  None Used                                */

/* The purpose of this module is to create a new module table element and    */
/* return a pointer to it. If there is not enough memory to malloc the new   */
/* element, the module will give the user an error message and exit the      */
/* program.                                                                  */

{                                                   /* module_alloc begin */
  struct mod_result *new_result;          /* pointer to new table element */
  new_result = (struct mod_result *)malloc(size);    /* malloc new module */
                                                     /*     table element */
  if (new_result == NULL)                        /* if no memory left ... */
    {
      fprintf(stderr, 
              "<ERROR> - INSUFFICIENT MEMORY : module_alloc\n<EXIT>\n");
      exit(2);                              /* print error and exit */
    }
  return new_result;                        /* return pointer to new element */
}                                           /* module_alloc end */



void load_mod_table(mod_tab_size,mod_table)
int mod_tab_size;
struct mod_result *mod_table[];

/******************************************************************************

               MODULE NAME       : load_mod_table
               CREATION DATE     : September 1, 1991
               LAST UPDATE       : November 20, 1991
               PROGRAMMER        : Roger Wynn
               UPDATE PROGRAMMER : Roger Wynn

******************************************************************************/


/*                              PARAMETER DICTIONARY
     2) mod_tab_size : The desired size of the module table. It is used to
                       create the module table with as many elements as are
                       needed.
     3) mod_table : The table used to store all the metric data for each of the
                    modules. It enters the module empty of module space and it
                    returns with enough space available to store the metrics
                    for all of the modules.


                                GLOBAL DICTIONARY
     1) FALSE : Boolean value of false that is defined in mas.globals.c. 
                It is used to initalize the boolean variables in the module
                table.
*/

/* The purpose of this module is to initailize the module table to it's 
   starting values.
*/

/* load_mod_table begin */
{
  int more;                        /* loop index */
  for (more = 0;more < mod_tab_size;more++) /* For all modules plus one  */
  { /*    mod_name[more] = malloc(sizeof(struct mod_list));	*/
    mod_table[more] = module_alloc(sizeof(struct mod_result)); 
    mod_table[more] -> percent_fit	      	= 0.0;
    mod_table[more] -> num_except       	= 0;
    mod_table[more] -> num_stmnts       	= 0;
    mod_table[more] -> LOC              	= 0;
    mod_table[more] -> vsubg            	= 0;
    mod_table[more] -> max_nest         	= 0;
    mod_table[more] -> num_rec_calls    	= 0;
    mod_table[more] -> fanin            	= 0;
    mod_table[more] -> fanout           	= 0;
    mod_table[more] -> ext_files        	= 0;
    mod_table[more] -> num_iocall		= 0;
    mod_table[more] -> unusedvar        	= 0;
    mod_table[more] -> global           	= 0;
    mod_table[more] -> num_params       	= 0;
    mod_table[more] -> num_local        	= 0;
    mod_table[more] -> num_welldecl		= 0;
    mod_table[more] -> ave_welldecl     	= 0.0;
    mod_table[more] -> percent_welldecl	        = 0.0;
    mod_table[more] -> num_initvars		= 0;
    mod_table[more] -> max_span		        = 0;
    mod_table[more] -> ave_span         	= 0.0;
    mod_table[more] -> percent_uncrowd_stmnts 	= 0.0;
    mod_table[more] -> header           	= FALSE;
    mod_table[more] -> horiz_sp    	  	= FALSE;
    mod_table[more] -> vert_sp 	  	        = FALSE;
    mod_table[more] -> indent           	= FALSE;
    mod_table[more] -> num_mean_vars    	= 0;
    mod_table[more] -> num_comments		= 0;
  } 
}                       /* load_mod_table end */



int big_enough(mod_tab_size)
int mod_tab_size;

/******************************************************************************
               MODULE NAME       : big_enough
               CREATION DATE     : September 1, 1991
               LAST UPDATE       : November 20, 1991
               PROGRAMMER        : Roger Wynn
               UPDATE PROGRAMMER : Roger Wynn
******************************************************************************/


/*                              PARAMETER DICTIONARY

     1) mod_tab_size : The size of the module table. It is used to determine if
                       the module table is big enough to accomidate all the
                       modules in the tested program.

                                GLOBAL DICTIONARY
     1) MOD_ARRAY : The maximum size of the module table that is defined in
                    mas.globals.c. It is used to compare with the size of the
                    module table needed for the tested program.
     2) TRUE : Boolean value of true that is defined in mas.globals.c. It is
               used to set the ok flag.
     3) FALSE : Boolean value of false that is defined in mas.globals.c. 
                It is used to set the ok flag.


   The purpose of this module is to determine if the MAS tables are big 
   enough to contain the data extracted from the tested program. This is 
   necessary to prevent array subscript errors and unintentional abortion 
   of the program. If this module aborts the test, it will inform the user 
   of the problem.

   This module is called from main
*/

{                                       /* big_enough begin              */
  int ok = TRUE;                        /* hope tables are big enough    */
  if (mod_tab_size > MOD_ARRAY)         /* if module table too small ... */
  {
    fprintf(stderr, 
          "<ERROR> - mod_table too small for program.\n Change MOD_ARRAY to ");
    fprintf(stderr, 
          "%d in mas.globals.c and recompile.\n<ABORT>\n", mod_tab_size);
    ok = FALSE;                          /* give error and abort */
  }
  return(ok);                            /* return T/F ok flag   */
}                                        /* big_enough end       */



main(argc, argv)
int argc;
char *argv[];

/****************************************************************************/
/*                                                                          */
/*               MODULE NAME       : main                                   */
/*               CREATION DATE     : September 1, 1991                      */
/*               LAST UPDATE       : December 15, 1991                      */
/*               PROGRAMMER        : Roger Wynn                             */
/*               UPDATE PROGRAMMER : Mark Vieselmeyer                       */
/*                                                                          */
/****************************************************************************/

/*                              PARAMETER DICTIONARY
     1) argc : The number of command line arguments presented. It is used to
               determine how many arguments to process.
     2) argv : A pointer to an array containing the command line arguments. It
               is used in processing the command line arguments.



                                GLOBAL DICTIONARY                           
                                 NOT APPLICABLE                             
                                                                            

   The purpose of this module is to drive the MAS system. It calls all the 
   other modules of the system as they are needed. It allows the user to enter
   commands on the command line or before each program test if no command line
   arguments are used.
*/


{                        /* main begin */

  int c;					/* a local char for options */
  int option_count = 0; /*lowther93: number of options detected */
  extern int optind, opterr, optopt;		/* more stuff got getopt */

  Intro();               /* MAS introduction */

/*lowther93: set delimiter to tab before loop to ensure the default is a tab */
  delimiter = '\t';

  while ( (c = getopt(argc, argv, "tTbBcC")) != EOF )
  {
    option_count++; /*lowther93: Count up all the arguments detected */
    switch (c) {
    case 't': case 'T':
      delimiter = '\t';
      break;
    case 'b': case 'B':
      delimiter = ' ';
      break;
    case 'c': case 'C':
      delimiter = ',';
      break;
    };
  };
  
  if(option_count > 1)
  {
    fprintf(stderr, "** Illegal parameter combination.\n\n");
    fprintf(stderr, "\tUsage: back [-t|-b|-c] [filename]\n\n");
    exit(-1);
  }

 /*lowther93: The last thing on the command line should be the filename.*/
 /*lowther93: Ensure that there is a default file name if one isn't supplied.*/

  if(strlen(argv[option_count+1]) == 0) 
    strcpy(inputfilename,DEFAULT_NAME);
  else
    strcpy(inputfilename, argv[option_count+1]);
  
  strcpy(outfilename, inputfilename);
  strcpy(output_file, inputfilename);

  /*lowther93: build the two output filenames used in met_dump.c*/

  strcat(outfilename, ".metrics");
  strcat(output_file, ".report");

  if (big_enough(MOD_ARRAY) == TRUE)
  {  /* if tables are big enough ... */
    
    load_mod_table(MOD_ARRAY, mod_table);
    GetData();
    CalcGlobal();
    calculate_metrics();
    PRINT_METRICS();

    /* DEA '93: new output to screen while running */
    fprintf(stdout," ... done\n\n\n");
  }
}   /* main end */
