/*$Header: met_dump.c,v 4.0 94/02/01 20:21:59 tedd Exp $*/

/************************************************************************
 *                                                                       *
 *        Metric Dump File for the MAS-CII Maintainability Tool          *
 *              CS 481 Senior Group Project - Fall 1992                  *
 *        Group Members : Craig Anderson, Kelly Hall, Vince Lowe,        *
 *                        Theresa Rhoades, Dayne Steed                   *
 *        Date File Created : September 1, 1992                          *
 *        Last File Update  : October 31, 1992                           *
 *                                                                       *
 ************************************************************************/

/**************************************************************************
  This sub-program is included in the MAS system as "met_dump.c" for the
  MAS system. "met_dump.c" produces the file(s) containing the overall
  program and individual module metrics generated by the system. The
  overall program metrics are written to a data file called "all_met.dat"
  and the module metrics are written to a data file called "mod_met.dat".
  
  For additional information on this sub-program and the data files it
  creates, as well as the MAS system, refer to the MAS Users Manual
  or the MAS Software Support Document.
***************************************************************************/

#include <stdio.h>

#include "globals.h"

/*  GLOBAL VARIABLE */
char metric_type;   /*  This variable marks each row in the metric dump 
                        as an overall or module metric */
FILE *metricfile;
FILE *reportfile;   /*lowther93: Out file for the metric report information. */

extern char* optarg;

void ALL_METRICS(all_metrics)
struct all_result all_metrics;

/**************************************************************************
       
       MODULE NAME       : ALL_METRICS
       CREATION DATE     : September 1, 1992
       LAST UPDATE       : October 12, 1992
       PROGRAMMER        : Theresa Rhoades
       UPDATE PROGRAMMER : Theresa Rhoades
       
****************************************************************************/
     
     
/*                              PARAMETER DICTIONARY
                                     
  1) all_metrics : The data structure that contains the overall program
     metrics. It arrives with all the metric data contained
     within it and is used to print the overall metrics dump.
  2) metricfile : This is the pointer to the output file to be written to. If
     there is no file to write to, it is NULL.
     
     The purpose of this procedure is to print the overall metrics dump. It
     uses a series of "fprintf" statements to produce a desirable format for
     the overall metrics contained in "all_met.dat". All numbers are printed
     in 6 spaces and the Real numbers have two of those spaces past the
     decimal. The end of the report is a summary that contains the PERCENTAGE
     OF OVERALL FIT. This is the percentage of maintainability that MAS
     determines for the processed program.
*/
     
{              /* all_report begin */

  /*  lowther93: Is this right? I think It's already divided 
   *  by mod_tab_size in calc.c.
   */

  /*  all_metrics.ave_per_uncrowd_stmnt /= mod_tab_size;*/
  
  metric_type = 'O';
    
  /****  SUMMARY:  Overall program summary ****/
  /* lowther93: removed the letter from each line.                  */
  /*		fprintf(metricfile, "%c%c", metric_type, delimiter);*/

  fprintf(metricfile, "%.2f%c", all_metrics. percent_overall_fit, delimiter);

  /*  modules with exceptions */  
  fprintf(metricfile, "%.2f%c", percent_mods_with_except, delimiter);

  /*  average exceptions per module */
  fprintf(metricfile, "%.2f%c", ave_exceptions, delimiter);


  /****  CONTROL:  Control Metrics for Overall Program ****/

  fprintf(metricfile, "%.2f%c", all_metrics. percent_tot_control_fit, delimiter);

  /*  Total number of modules */
  fprintf(metricfile, "%d%c", all_metrics. tot_mods, delimiter);

  /*  Total lines of code */
  fprintf(metricfile, "%d%c", all_metrics. tot_LOC, delimiter);

  /*  Maximum lines of code in a module */
  fprintf(metricfile, "%d%c",	all_metrics. max_LOC, delimiter);

  /*  Average lines of code per module */
  fprintf(metricfile, "%.2f%c",	all_metrics. ave_LOC, delimiter);

  /* sm Total NCSS sm */
  fprintf(metricfile, "%d%c", all_metrics. tot_NCSS, delimiter);

  /* sm Max NCSS in a module sm */
  fprintf(metricfile, "%d%c", all_metrics. max_NCSS, delimiter);

  /* sm Average NCSS per module sm */
  fprintf(metricfile, "%.2f%c", all_metrics. ave_NCSS, delimiter);

  /*  Total program v(g) */
  fprintf(metricfile, "%d%c",	all_metrics. tot_vsubg, delimiter);

  /*  Maximum module v(g) */
  fprintf(metricfile, "%d%c",	all_metrics. max_vsubg, delimiter);

  /*  Average module v(g) */

  fprintf(metricfile, "%.2f%c", all_metrics. ave_vsubg, delimiter);
  fprintf(metricfile, "%.2f%c", all_metrics. tot_Halv, delimiter);
  fprintf(metricfile, "%.2f%c", all_metrics. max_Halv, delimiter);
  fprintf(metricfile, "%.2f%c", all_metrics. ave_Halv, delimiter);
  fprintf(metricfile, "%.2f%c", all_metrics. tot_Hale, delimiter);
  fprintf(metricfile, "%#6.2f%c", all_metrics. max_Hale, delimiter);
  fprintf(metricfile, "%#6.2f%c", all_metrics. ave_Hale, delimiter);

  /*  Maximum fan-out */
  fprintf(metricfile, "%d%c",	all_metrics. max_fanout, delimiter);

  /*  Maximum Fan-in */
  fprintf(metricfile, "%d%c",	all_metrics. max_fanin, delimiter);

  /*  Average Fan-in */
  fprintf(metricfile, "%.2f%c",	all_metrics. ave_fanin, delimiter);

  /*  Maximum nesting in a module */
  fprintf(metricfile, "%d%c",	all_metrics. max_nesting, delimiter);

  /*  Average module nesting */
  fprintf(metricfile, "%.2f%c",	all_metrics. ave_nesting, delimiter);

  /*  Maximum recursive calls in a module */
  fprintf(metricfile, "%d%c",	all_metrics. max_rec_call, delimiter);

  /*  Number of recursive modules */
  fprintf(metricfile, "%d%c",	all_metrics. tot_rec_mod, delimiter);

  
  /****  DATA FLOW:  Data Flow Metrics for Overall Program ****/

  fprintf(metricfile, "%.2f%c", all_metrics. percent_tot_data_fit, delimiter);

  /*  Total program variables */
  fprintf(metricfile, "%d%c", all_metrics. tot_vars, delimiter);

  /*  Total local variables */
  fprintf(metricfile, "%d%c", all_metrics. tot_local, delimiter);

  /*  Total global variables */
  fprintf(metricfile, "%d%c", all_metrics. tot_global, delimiter);	

  /*  Total unused variables */
  fprintf(metricfile, "%d%c", all_metrics. tot_unusedvar, delimiter);

  /* average number of parameters or arguments*/
  fprintf(metricfile, "%.2f%c", all_metrics. ave_num_params, delimiter);

  /*  percentage of total initialized variables */
  fprintf(metricfile, "%.2f%c", all_metrics. percent_tot_init_vars, delimiter);

  /*  Total external Files  */
  fprintf(metricfile, "%d%c", all_metrics. tot_ext_files, delimiter);

  /*  Average number of IO calls per module */
  fprintf(metricfile, "%.2f%c", all_metrics. ave_num_iocalls, delimiter);

  /*  Average variables span over modules */
  fprintf(metricfile, "%.2f%c", all_metrics. ave_var_span, delimiter);

  /*  maximum variable span in a module */
  fprintf(metricfile, "%d%c", all_metrics. max_var_span, delimiter);

  /* percent well declared vars */
  fprintf(metricfile, "%.2f%c", all_metrics. percent_welldecl, delimiter);

  /*  Number of preprocessor directives */
  fprintf(metricfile, "%d%c", all_metrics. num_prep, delimiter);

  /*  Lowther93:  fprintf(metricfile, "%d%c", total_welldecl, delimiter); 
   *  total well declared vars 
   */

  /*  lowther93:  fprintf(metricfile, "%.2f%c",
   *  all_metrics. ave_welldecl, delimiter); 
   *  average of well declared variables 
   */

  /*  fprintf(metricfile, "%.2f%c",   all_metrics. ave_var_mod, delimiter);
      average variables per module */
  /*  fprintf(metricfile, "%d%c", all_metrics. max_var_mod, delimiter);
      maximum variables in a module */


  /****  TYPOGRAPHIC: Typographic Metrics for Overall Program ****/

  fprintf(metricfile, "%.2f%c", all_metrics. percent_tot_typo_fit, delimiter);

  /*  Percentage of modules with headers */
  fprintf(metricfile, "%.2f%c", all_metrics. percent_mod_with_header, delimiter);

  /*  Percentage of modules with comments */
  fprintf(metricfile, "%.2f%c", all_metrics. percent_mods_with_comments, 
          delimiter);

  /*  Percent of modules with horizontal spacing */
  fprintf(metricfile, "%.2f%c", all_metrics. percent_mods_with_horiz_sp,
          delimiter);

  /*  Percent of modules with vertical spacing */
  fprintf(metricfile, "%.2f%c", all_metrics. percent_mods_with_vert_sp,
          delimiter);

  /* Percent of modules with identation */
  fprintf(metricfile, "%.2f%c", all_metrics. percent_mods_with_indent,
          delimiter);

  /*  average percent of uncrowded statements */
  fprintf(metricfile, "%.2f%c", all_metrics. ave_per_uncrowd_stmnt,
          delimiter);

  /*  average percent of comments per module */
  fprintf(metricfile, "%.2f%c", all_metrics. ave_percent_comments,
          delimiter);

  /* average number of meaningful vars */
  fprintf(metricfile, "%.2f\n",   all_metrics. ave_per_mean_vars);			
  
/*  lowther93:
 *  fprintf(metricfile, "%d%c",   all_metrics. mod_with_comments, delimiter); 
 *  fprintf(metricfile, "%f%c",   all_metrics. ave_comments_per_mod, delimiter);
 *  fprintf(metricfile, "%d%c",   all_metrics. mod_with_vert_sp, delimiter);	
 *  fprintf(metricfile, "%d%c",   all_metrics. mod_with_horiz_sp, delimiter);
 *  fprintf(metricfile, "%d%c",   all_metrics. mod_with_indent, delimiter);  
 *  fprintf(metricfile, "%d%c", 	all_metrics. tot_uncrowd_stmnt, delimiter);
 *  fprintf(metricfile, "%f%c",   all_metrics. ave_uncrowd_stmnt, delimiter);	
 */

}   /* all_metrics end */



void MOD_METRICS()
/**********************************************************************
 *
 *               MODULE NAME       :  MOD_METRICS
 *               CREATION DATE     : September 1, 1992
 *               LAST UPDATE       : October 12, 1992
 *               PROGRAMMER        : Theresa Rhoades
 *               UPDATE PROGRAMMER : Theresa Rhoades
 *
 ***********************************************************************/
     
     
/************************************************************************
       PARAMETER DICTIONARY
       
       1) metricfile : 	This is the pointer to the output file to be written to. If
       there is no file to write to, it is NULL.
       without forcing the user to enter all the module numbers.
       2) mod_tab_size : This is the number of modules in the module table. It is
       used to make sure that the module numbers entered exists
       and it is used to enter all the modules into the module
       numbers array if they are requested by the user.
       3) mod_table : This is the module table containing all the metrics for
       every module tested. This is used to print the metrics for
       the modules that will be requested by the user.
****************************************************************************/
     
/***************************************************************************
       GLOBAL DICTIONARY
       
       1) stop : This is a T/F flag indicating if a user wants to stop. It is
       defined in "globals.c" and is used here to indicate if the user
       wants to stop looking at the module report.
       2) TRUE : This is a boolean value defined in "globals.c". It is used here
       with the stop and the not_normal flags.
       3) FALSE : This is a boolean value defined in "globals.c". It is used here
       with the not_normal flag.
*****************************************************************************/
     
     
/****************************************************************************
       The purpose of this procedure is to print the module metric report. It
       uses a series of "fprintf" statements to produce a desirable format for
       the module metrics contained in "mod_table". All numbers are printed in
       6 spaces and the Real numbers have two of those spaces past the decimal.
       This module prompts the user for the numbers of the modules he wants and
       gives him the option to break out of the report at the end of each module.
       The user also is given the ability to enter only one number to get all of
       the modules to print or to have only the modules that were found to have
       exceptions to print. If there are too many modules listed, all modules
       will print.
****************************************************************************/
     
{

/*lowther93:  char c; */

  int more;                /* index for module print and exceptions */
  int number = 1;

/*lowther93:  char trace = ',';   looks for ",  " in users module list */
/*lowther93:  metric_type = 'M'; */

/* print all modules desired to file */
  for (more = 0;more < mod_tab_size;more++)  
  {	
    fprintf(metricfile, "%d%c", number++, delimiter);   
    fprintf(metricfile, "%s%c",   mod_table[more] -> name, delimiter);

    /* Percentage of fit */   
    fprintf(metricfile, "%.2f%c",mod_table[more] -> percent_fit, delimiter);
    
    /*  number of exceptions */
    fprintf(metricfile, "%d%c",   mod_table[more]->num_except, delimiter);

    /****  CONTROL:  Module Control Metrics ****/
    
    fprintf(metricfile, "%.2f%c",   mod_table[more] -> percent_control_fit,
            delimiter);
    fprintf(metricfile, "%d%c",   mod_table[more] -> LOC, delimiter);
    fprintf(metricfile, "%d%c",   mod_table[more] -> NCSS, delimiter);
    fprintf(metricfile, "%d%c",   mod_table[more] -> vsubg, delimiter);
    fprintf(metricfile, "%d%c",   mod_table[more] -> max_nest, delimiter);

    /*  Maximum Recursive calls */
    fprintf(metricfile, "%d%c",   mod_table[more] -> num_rec_calls, delimiter);
    fprintf(metricfile, "%d%c",   mod_table[more] -> fanout, delimiter);
    fprintf(metricfile, "%d%c",   mod_table[more] -> fanin, delimiter);
    fprintf(metricfile, "%.2f%c",   mod_table[more] -> Halv, delimiter);
    fprintf(metricfile, "%.2f%c",   mod_table[more] -> Hale, delimiter); 

    /****  DATA FLOW:  Module Data Flow Metrics ****/

    fprintf(metricfile, "%.2f%c",   mod_table[more] -> percent_data_fit,
            delimiter);

    /*  Number of external files */
    fprintf(metricfile, "%d%c",   mod_table[more] -> ext_files, delimiter);

    /*  Number of local variables */
    fprintf(metricfile, "%d%c",   mod_table[more] -> num_local, delimiter);

    /*  number of global variables */
    fprintf(metricfile, "%d%c",   mod_table[more] -> global, delimiter);

    /* Number of unused variables */
    fprintf(metricfile, "%d%c",   mod_table[more] -> unusedvar, delimiter);

    /*  Number of formal parameters */
    fprintf(metricfile, "%d%c",   mod_table[more] -> num_params, delimiter);
 
    fprintf(metricfile, "%d%c",   mod_table[more] -> num_iocall, delimiter);

    /* percentage initialized vars */
    fprintf(metricfile, "%.2f%c",   mod_table[more] -> percent_initvars,
            delimiter);

    /*  percent well declared vars */
    fprintf(metricfile, "%.2f%c",   mod_table[more] -> percent_welldecl,
            delimiter);
    
    /*  average variables span */
    fprintf(metricfile, "%.2f%c",   mod_table[more] -> ave_span, delimiter);

    /*  maximum variable span */
    fprintf(metricfile, "%d%c",   mod_table[more] -> max_span, delimiter);

    /*lowther93:    
      fprintf(metricfile, "%d%c",   mod_table[more] -> num_welldecl, delimiter);  
    */
    
    /*lowther93:    
      fprintf(metricfile, "%.2f%c", mod_table[more] -> ave_welldecl, delimiter);
    */
    
    /*lowther93:    
      fprintf(metricfile, "%d%c",    mod_table[more] ->  num_initvars, delimiter);
    */
    
    /**** TYPOGRAPHIC :  Module Typographic Metrics ****/
    
    fprintf(metricfile, "%.2f%c", mod_table[more]-> percent_typo_fit, delimiter);
    fprintf(metricfile, "%.2f%c", mod_table[more] ->  percent_comments, 
            delimiter);
    fprintf(metricfile, "%.2f%c", mod_table[more] -> percent_uncrowd_stmnts,
            delimiter);
    fprintf(metricfile, "%.2f%c", mod_table[more] -> percent_meaningful,
            delimiter);
    fprintf(metricfile, "%d%c", mod_table[more] -> header, delimiter);
    fprintf(metricfile, "%d%c", mod_table[more] -> horiz_sp, delimiter);
    fprintf(metricfile, "%d%c", mod_table[more] -> vert_sp, delimiter); 
    fprintf(metricfile, "%d\n", mod_table[more] -> indent); 

    /*  lowther93:
     *  fprintf(metricfile, "%d%c",   mod_table[more] -> num_uncrowd_stmnt, delimiter);
     *  fprintf(metricfile, "%d%c",   mod_table[more] -> num_mean_vars, delimiter);
     *  fprintf(metricfile, "%d%c",   mod_table[more] -> num_comments, delimiter);
     */

  } /* end for  */
}


  /*lowther93: Prints the overall report page. */

void print_overall_report(struct all_result all_metrics)
{
  fprintf(reportfile,"\n\n");    /* same report written to output file */
  
  /* Lowther92:find the name of the source file and place it here*/
 
  fprintf(reportfile,"\t\t\t%s\n", inputfilename);
  fprintf(reportfile,"\t\t\tOVERALL REPORT\n\n");
  
  /* Lowther92:	fprintf(reportfile,"\t\t\t  **** SUMMARY ****\n\n");*/
  
  fprintf(reportfile,"SUMMARY:\n");
  fprintf(reportfile,"    AVERAGE PERCENT FIT          : %6.2f\n", 
          all_metrics. percent_overall_fit);
  fprintf(reportfile,"    %% Modules with Exceptions    : %6.2f\n", 
          percent_mods_with_except);
  fprintf(reportfile,"    Ave. Exceptions per Module   : %6.0f\n", 
          ave_exceptions);
  
  /* control dimension for output file */

  fprintf(reportfile,"\n\nCONTROL FLOW FIT:\t%6.2f\n", 
          all_metrics. percent_tot_control_fit );
  fprintf(reportfile,"  Total Modules : %6d\n", all_metrics.tot_mods);
  
  fprintf(reportfile,"  Total LOC     : %6d\t", all_metrics.tot_LOC);
  fprintf(reportfile,"  Max LOC     : %6d\t", all_metrics.  max_LOC);
  fprintf(reportfile,"  Ave. LOC     : %6.0f\n", all_metrics.  ave_LOC);
  fprintf(reportfile,"  Total NCSS    : %6d\t", all_metrics.tot_NCSS);
  fprintf(reportfile,"  Max NCSS    : %6d\t", all_metrics.  max_NCSS);
  fprintf(reportfile,"  Ave. NCSS    : %6.0f\n", all_metrics.  ave_NCSS);
  fprintf(reportfile,"  Total V(g)    : %6d\t", all_metrics.tot_vsubg);
  fprintf(reportfile,"  Max V(g)    : %6d\t", all_metrics. max_vsubg);
  fprintf(reportfile,"  Ave. V(g)    : %6.0f\n", all_metrics. ave_vsubg);
  fprintf(reportfile,"  Total Hal's V : %6.0f\t",all_metrics. tot_Halv);
  fprintf(reportfile,"  Max Hal's V : %6.0f\t", all_metrics. max_Halv);
  fprintf(reportfile,"  Ave Hal's V  : %6.0f\n", all_metrics. ave_Halv);
  fprintf(reportfile,"  Total Hal's E : %6.0f\t", all_metrics.tot_Hale);
  fprintf(reportfile,"  Max Hal's E : %6.0f\t", all_metrics.max_Hale);
  fprintf(reportfile,"  Ave Hal's E  : %6.0f\n\n", all_metrics.ave_Hale);
  fprintf(reportfile,"  Max Fan-out   : %6d\t", all_metrics. max_fanout);
  fprintf(reportfile,"  Max Fan-in  : %6d\t", all_metrics.  max_fanin);
  fprintf(reportfile,"  Ave. Fan-in  : %6.2f\n", all_metrics. ave_fanin);
  fprintf(reportfile,"  Max Nesting   : %6d\t\t\t\t", all_metrics. max_nesting);
  fprintf(reportfile,"  Ave. Nesting : %6.0f\n", all_metrics. ave_nesting);
  fprintf(reportfile,"  Max Recursive Calls in a Module : %6d\n", 
          all_metrics. max_rec_call );
  fprintf(reportfile,"  Number of Recursive Modules     : %6d\n",
          all_metrics.  tot_rec_mod);
  
  /* data dimension for output file */  

  fprintf(reportfile,"\n\nDATA FLOW FIT:\t\t%6.2f\n", 
          all_metrics. percent_tot_data_fit);
  fprintf(reportfile,"  Total Vars   : %6d\t", all_metrics. tot_vars);
  fprintf(reportfile,"  Total Local  : %6d\n", all_metrics. tot_local);
  fprintf(reportfile,"  Total Global : %6d\t", all_metrics. tot_global);
  fprintf(reportfile,"  Total Unused : %6d\n", all_metrics.  tot_unusedvar);
  
  /*lowther93: average number of arguments (or parameters) in the system */

  fprintf(reportfile,"  Average Number of Arguments          : %6.0f\n", 
          all_metrics. ave_num_params); 
  fprintf(reportfile,"  %% Tot Init Vars                      : %6.0f\n", 
          all_metrics. percent_tot_init_vars);

  /*  Lowther92:	
   *
   *  fprintf(reportfile,"  Max vars in a mod               : %6d\t",
   *  all_metrics.); 
   */

  /*  Lowther92:   
   *  fprintf(reportfile,"  Ave vars per mod  : %6.0f\n", all_metrics.); 
   */

  /*  Lowther92:	
   *  fprintf(reportfile,"  Total well declared vars         : %6d\n",
   *  all_metrics.);
   */

  /*  Lowther92:	
   *  fprintf(reportfile,"  Ave Well-Declared Variables      : %6.2f\n",
   *  all_metrics.);
   */

  fprintf(reportfile,"  Total Number of External Files       : %6d\n",
          all_metrics. tot_ext_files);
  fprintf(reportfile,"  Ave. Number of I/O Calls per Module  : %6.0f\n",
          all_metrics. ave_num_iocalls);
  fprintf(reportfile,"  Ave. Variable Span per Module        : %6.0f\n",
          all_metrics. ave_var_span);
  fprintf(reportfile,"  Max  Variable Span                   : %6d\n",
          all_metrics. max_var_span );
  fprintf(reportfile,"  Total %% Well-Declared Variables      : %6.0f\n",
          all_metrics. percent_welldecl );
  fprintf(reportfile,"  # of Preprocessor Directives         : %6d\n",
          all_metrics. num_prep );
  
  /* typographics  for output file */
    
  fprintf(reportfile,"\n\nTYPOGRAPHIC FIT:\t%6.2f\n", 
          all_metrics. percent_tot_typo_fit );

  /* Lowther92: Replaced mod_with_header with percent_mod_with_header */

  fprintf(reportfile,"  %% Modules with Headers                      : %6.0f\n", 
          all_metrics. percent_mod_with_header);

  /*  Lowther92:	
   *  fprintf(reportfile,"  # Modules with Comments : %6d\n", all_metrics. );
   */
  
  fprintf(reportfile,"  %% Modules with Comments                     : %6.0f\n",
          all_metrics. percent_mods_with_comments );
  fprintf(reportfile,"  %% Mods with Horiz Spacing                   : %6.0f\n",
          all_metrics. percent_mods_with_horiz_sp );
  fprintf(reportfile,"  %% Mods with Vertical Spacing                : %6.0f\n",
          all_metrics. percent_mods_with_vert_sp );
  fprintf(reportfile,"  %% Modules with Indentation                  : %6.0f\n",
          all_metrics.percent_mods_with_indent );

  /*  Lowther92:	
   *  fprintf(reportfile,"  Ave comments per module             : %6.0f\n",
   *  all_metrics.  );
   */
  
  fprintf(reportfile,"  Ave. %% Uncrowded Statements within a module : %6.0f\n",
          all_metrics.ave_per_uncrowd_stmnt );
  fprintf(reportfile,"  Ave. %% comments within a module             : %6.0f\n",
          all_metrics. ave_percent_comments );
  fprintf(reportfile,"  Ave. %% Meaningful Vars within a module      : %6.0f\n",
          all_metrics.  ave_per_mean_vars);

  /*  Lowther92:	
   *  fprintf(reportfile,"  # Mods with Vertical Spacing   : %6d\n", all_metrics.);
   */
  
  /*  Lowther92:	
   *  fprintf(reportfile,"  # Mods with Horizontal Spacing : %6d\n", all_metrics.);
   */

  /*  Lowther92:	
   *  fprintf(reportfile,"  # Mods with Indentation        : %6d\n", all_metrics.);
   */
  
  /*  Lowther92:
   *  fprintf(reportfile,"  Tot Uncrowd Statements         : %6d\n",
   *  tot_uncrowd_stmnt);
   */
  
  /*  Lowther92:	
   *  fprintf(reportfile,"  Ave Uncrowd Statement          : %6.0f\n",
   *  all_metrics.y  );
   */
  
  fprintf(reportfile,"\f");
}

/* Lowther92: Little macro to convert boolean to a yes or no string. */

#define MACRO_YN(x) ((x)?("Yes"):("No"))

  /*  lowther93: Prints the individual module report pages. */

void print_module_report()
{
  int more, number = 1;

  for (more = 0;more < mod_tab_size;more++) /* print all modules desired to file */
  {	
    fprintf(reportfile, "\n\n");
    fprintf(reportfile, "\t\t %s MODULE REPORT\n", mod_table[more] -> name);
    fprintf(reportfile, "\t\t%6.2f AVERAGE FIT\n\n",
            mod_table[more] -> percent_fit);
    fprintf(reportfile, "Metrics for Module # 	: %6d\n", number++);

    /*  Lowther92:	
     *  fprintf(reportfile, "Module Name 		: %s\n",  module_name);
     */

    fprintf(reportfile, "Number of Exceptions 	: %6d\n",
            mod_table[more] -> num_except);

    /* control dimension for module report */

    fprintf(reportfile, "\n\nCONTROL FLOW FIT :\t\t\t\t%6.2f\n", 
            mod_table[more] -> percent_control_fit);
    fprintf(reportfile, "  LOC                     : %6d\n",
            mod_table[more] -> LOC);
    fprintf(reportfile, "  NCSS                    : %6d\n",
            mod_table[more] -> NCSS);
    fprintf(reportfile, "  V(g)                    : %6d\n",
            mod_table[more] -> vsubg);
    fprintf(reportfile, "  Max Nesting             : %6d\n",
            mod_table[more] -> max_nest);
    fprintf(reportfile, "  # Recursive Calls       : %6d\n",
            mod_table[more] -> num_rec_calls);
    fprintf(reportfile, "  Fan-out                 : %6d\n",
            mod_table[more] -> fanout);
    fprintf(reportfile, "  Fan-in                  : %6d\n",
            mod_table[more] -> fanin);
    fprintf(reportfile, "  Halstead's Vol          : %6.0f\n",
            mod_table[more] -> Halv);
    fprintf(reportfile, "  Halstead's E            : %6.0f\n",
            mod_table[more] -> Hale);
    
    /* data dimension for module report */

    fprintf(reportfile, "\n\nDATA FLOW FIT:\t\t\t\t\t%6.2f\n",
            mod_table[more] -> percent_data_fit);
    fprintf(reportfile, "  # External Files        : %6d\n",
            mod_table[more] -> ext_files);
    fprintf(reportfile, "  # Local Variables       : %6d\n",
            mod_table[more] -> num_local);
    fprintf(reportfile, "  # Global Variables      : %6d\n",
            mod_table[more] -> global);
    fprintf(reportfile, "  # Unused Variables      : %6d\n",
            mod_table[more] -> unusedvar);
    fprintf(reportfile, "  # Formal Arguments      : %6d\n",
            mod_table[more] -> num_params);
    fprintf(reportfile, "  # I/O Calls             : %6d\n",
            mod_table[more] -> num_iocall);
    fprintf(reportfile, "  %% Initialized Vars      : %6.2f\n", 
            mod_table[more] -> percent_initvars);
    fprintf(reportfile, "  %% Well-Declared Vars    : %6.2f\n", 
            mod_table[more] -> percent_welldecl);
    fprintf(reportfile, "  Ave. Var Span           : %6.2f\n",  
            mod_table[more] -> ave_span);
    fprintf(reportfile, "  Max. Var Span           : %6d\n",    
            mod_table[more] -> max_span);
    
    /*  Lowther92:	
     *  fprintf(reportfile, "  # Well Declared : %6d\n",    num_welldecl);
     */
    
    /*  Lowther92:	f
     *  printf(reportfile, "  # Init Vars     : %6d\n",    num_initvars);
     */
    
    /* typographic dimension for module report */
    
    fprintf(reportfile, "\n\nTYPOGRAPHIC FIT :\t\t\t\t%6.2f\n", 
            mod_table[more] -> percent_typo_fit);    
    fprintf(reportfile, "  %% Comments              : %6.2f\n", 
            mod_table[more] -> percent_comments);
    fprintf(reportfile, "  %% Uncrowded Statements  : %6.2f\n", 
            mod_table[more] -> percent_uncrowd_stmnts);
    fprintf(reportfile, "  %% Meaningful Variables  : %6.2f\n",    
            mod_table[more] -> percent_meaningful); 
    fprintf(reportfile, "  Header Comment          : %6s\n",    
            MACRO_YN(mod_table[more] -> header));
    fprintf(reportfile, "  Horizontal Spacing Used : %6s\n",    
            MACRO_YN(mod_table[more] -> horiz_sp));
    fprintf(reportfile, "  Vertical Spacing Used   : %6s\n",    
            MACRO_YN(mod_table[more] -> vert_sp));
    fprintf(reportfile, "  Indentation Used        : %6s\n",    
            MACRO_YN(mod_table[more] -> indent));
    fprintf(reportfile, "\f");
    
    /*  Lowther92:	
     *  fprintf(reportfile, "  Number of Comments             : %6d\n", 
     *  num_comments);
     */
    
    /*  Lowther92:	
     *  fprintf(reportfile, "  Number of Uncrowded Statements : %6d\n", 
     *  num_uncrowd);
     */
    
    /*  Lowther92:	
     *  fprintf(reportfile, "  Number of Meaningful Variables : %6d\n",    
     *  num_mean_vars);
     */
    
  }
  fprintf(reportfile,"\n");
}  

void PRINT_METRICS()
{
  /* lowther93: open the filenames created in main.c */

  metricfile = fopen(outfilename, "w");   /* filename.metric */
  reportfile = fopen(output_file, "w");   /* filename.report */

  ALL_METRICS(all_metrics);
  print_overall_report(all_metrics);
  MOD_METRICS();
  print_module_report();
  fclose(metricfile);
  fclose(reportfile);
}


