/*$Header: sp-code.c,v 4.1 94/02/02 16:36:03 tedd Exp $*/

#include "sp-code.h"
#include "hash.h"

/*  TED '94--Removed some global definitions from the sp-code.h
 *           header file into this file below.
 */

static char *source;
static float tot_span, max_span;
static int tot_mods;
struct module_info *GLOBAL = NULL, *LAST_MODULE = NULL, *CURRENT_MODULE = NULL;
struct link_list *start_list = NULL, *end_list = NULL, *user_list;

extern unsigned int in_stmt;

/* TED '94--Used in loc_check to help count union lines. */

#define CODE 0
#define COM 1

/* DEA '93:  The "header" is used to flag if a module has a header.  
 *           The "comment" is used to count the # of comment lines.
 *           The "header_lines" is used to count the # of header lines.
 *           All are Globals.  */

int header = 0;
int header_lines = 0;
int comment = 0;

void external_file()
{  CURRENT_MODULE->num_of_external_files++;
}

/*    vertical_spacing
   
      called by : new_line.
   
      This procedure increments the vertical spacing count in the current
      module.
*/

void vertical_spacing()
{
  CURRENT_MODULE->has_vertical_spacing = 1;
}


/*    horizontal_spacing
            
      called by :  blank.
                 
      This procedure increments the horizontal spacing count in the
      current module.
*/

void horizontal_spacing()
{
  CURRENT_MODULE->has_horizontal_spacing = 1;
}

/*    lines_of_code
      
      called by : new_line.
      
      This procedure increments the lines of code counter in the current
      module.
*/

void lines_of_code()
{
  CURRENT_MODULE->lines_of_code++;
}


/*    indentation
        
      called by : newline.
        
      this procedure sets the indentation flag for the current module.
*/

void indentation()
{
  CURRENT_MODULE->is_indented = 1;
}

/*    nesting
        
      called by : lex, yacc.
        
      This procedure increments the nesting count on a '{' and decrements
      the count on a '}'.  If the count exceeds the current module - max
      nesting number the max nesting is updated.
*/

void nesting( char chr )
{
  static current_nesting = 0;
  struct module_info *temp = CURRENT_MODULE;
  
  if( temp == GLOBAL )
    return;
  
  while (temp->code_block != GLOBAL )
    temp = temp->code_block;
  
  if( chr == '}' )
    current_nesting--;
  else
    if( ++current_nesting > temp->max_nesting )
      temp->max_nesting = current_nesting;
}

/*   count_keywords
     
     called by : lex.
     
     This procedure increments the number of keywords in the current
     module. V(G).
*/

void count_keywords()
{
  CURRENT_MODULE->num_of_keywords++; 
}

/*    statement
         
      called by : lex.
         
      This procedure checks for crowded statement, and counts the number
      of crowded statments in the current_module. The number of uncrowded
      statements are computed by subtracting the number of crowded
      statements from the total number of statement.
*/

void statement()
{
  static last_line_seen = -1;
  if( last_line_seen == yylineno )
    CURRENT_MODULE->num_of_uncrowded_statements++;
  else
    last_line_seen = yylineno;
  
  CURRENT_MODULE->num_of_statements++;
  
  header = 0;   /* why are these here - DEA '94 */
  comment = 0;
}

/*    comment_lines
      
      called by : comment.
      
      This procedure increments the number of comment lines and sets a
      header if the comment is a possible header or increments the 
      header flag if the comment is a header.
*/

void comment_lines( int lines )
{
  static int last_line = -1;
  
  if( yylineno == (last_line+1) )
  {
    comment += lines;
  }
  else
    comment = lines;
  
  if(comment > 2)
  {
    if(where_in_code == WHERE_PROTO || where_in_code == WHERE_P_DEF
       || where_in_code == WHERE_I_DEC)
      CURRENT_MODULE->contains_comment_headers = 1;

    if( where_in_code == WHERE_GLOBAL )
      header = 1;
  }
  
  /* DEA '93: This counts all the header comment lines
   *          for each module. 
   */  
  
  if (CURRENT_MODULE == GLOBAL)
  {
    header_lines += lines;
  }

  /* TED '94--Added check to determine if comment is on a new line and
   *	      moved last_line = yylineno; to end of function 
   */
  if(yylineno != last_line)
    CURRENT_MODULE->num_of_comment_lines += lines;

  CURRENT_MODULE->lines_of_code += lines - 1;


  /* TED '94--If a comment starts on the first line of a file, then lines of
   *	      code will be off by one.  Corection below.
   */
  if(yylineno-lines-1 == -1)
  {
    ++CURRENT_MODULE->lines_of_code;
  }

  last_line = yylineno;
}


/*   add_operator
     
     called by : lex.
     
     This procedure finds the number that is associated with the
     inputted operator and increments that array element in the
     operators array.
*/

void add_operator( char *string )
{
  char *operators[OPERATOR_SIZE] =
  {"!","!=","%","%=","&","&&","&=","(","*","*=","+","++",
     "+=",",","-","--","-=","->",".","/","/=",":",";","<","<<","<<=",
     "<=","=","==",">",">=",">>",">>=","?","BREAK","CASE","CONTINUE",
     "DEFAULT","DO","ELSE","FOR","GOTO","IF","RETURN","SIZEOF","SWITCH",
     "WHILE","[","^","^=","{","|","|=","||","~"};
  int low = 0, high = OPERATOR_SIZE, mid = OPERATOR_SIZE/2;
  int equal = 0;
  
  for( equal = strcmp( string, operators[mid] );
      equal && high-low>1;
      equal = strcmp( string, operators[mid] ) )
  {
    if( equal > 0 )
    {
      low = mid;
      mid = (high+low)/2;
    }
    else if( equal < 0 )
    {
      high = mid;
      mid = (high+low)/2;
    }
  }
  
  if( equal == 1 )
  {
    mid = 0;
    equal = strcmp( string, operators[mid] );
  }
  
  if( equal == 0 )
  {
    CURRENT_MODULE->operators[mid]++;
  }
  else
  {
    fprintf(stderr, "ERROR in add_operators procedure...\n");
    fprintf(stderr, "\tString \"%s\" was sent and is not an operator.\n", string);
    fprintf(stderr, "\tIt was found on line %d\n",yylineno);
    fprintf(stderr, "\t***Abnormal exit***\n");
    exit(-1);
  }
}



/*    statement_list_identifiers
               
      called by : yacc.
               
      This procedure takes what is in the linked list and changes the
      identifiers in the identifier tree to hold the new information.  if
      the IS_PROCEDURE_CALL is equal to 1 then the first identifier in
      the list is a procedure name and is passed to check_procedures.
*/

void statement_list_identifiers()
{

/* lowther93: this is a bug fix from lanphi872@snake.cs.uidaho.edu */
/*  struct link_list *temp = start_list->next_node; */

  struct link_list *temp;

  if(start_list)
    temp = start_list->next_node;

/* lowther93: end bug fix */
  
  if( IS_PROCEDURE_CALL && start_list )
  {
    char *string = start_list->information->id_name;

    /* add to identifier tree */

    funcname_install( string );
    check_procedures( string );
    free_link( start_list );
    start_list = temp;
    IS_PROCEDURE_CALL--;
  }
  
  while( start_list )
  {
    change_identifier_tree();
        
    temp = start_list->next_node;
    free_link( start_list );
    start_list = temp;
  }
  
  start_list = end_list = NULL;
  
}


/*   add_declared_identifiers
                
     called by : yacc.
                
     This procedure takes what is in the linked list and adds those
     identifiers in the identifiers tree.  The identifiers in the list
     when this procedure is called will be only declared identifiers.
*/


void add_declared_identifiers()
{
  struct link_list *temp = start_list;

  while(temp)
  {
    add_identifier_tree();
    if( temp->information->initialized )
      CURRENT_MODULE->num_of_well_declared_identifiers++;
    
    start_list = temp;
    temp = temp -> next_node;
    free_link( start_list );
    start_list = temp;
  }
  end_list = start_list = NULL;
}


/*     add_to_list
       
       called by : yacc.
       
       This procedure takes what is in the yyidentifier string and places
       in onto the end of the list with the line it was found on.  If the
       GLOBAL_INITIALIZER is set to 1 then the identifier in yyidentifier
       was found in the declarations but should not be a declared
       identifiers (eg MAX is not a declared identifier in the line
       int array[MAX], ... ).
*/

void add_to_list()
{
  if(gab == 1 && in_stmt == 0)
    return;
  
  if(end_list)
  {
    end_list->next_node = malloc_link();
    end_list = end_list -> next_node;
  }
  else
    start_list = end_list = malloc_link();
  end_list ->next_node = NULL;
  end_list ->information->id_name = yyid_copy();
  end_list -> information->first_line_seen = yylineno;
  
}
/*    initialize_id
                 
      called by : yacc.
                 
      This procedure sets the initialized flag for the last item in the 
      linked list.  This item will be the last read identifiers from the
      stream.
*/

void initialize_id()
{
  if( end_list )
    end_list->information->initialized = 1;
  else
    printf(" not found \n");
}


/*    set_up_modules
                 
      called by : main.
                 
      this procedure initially sets up the global and other modules.
*/

void set_up_modules()
{
  GLOBAL = malloc_module();
  GLOBAL ->module_name = "Global";
  CURRENT_MODULE = LAST_MODULE = GLOBAL;
  WHERE( "set_up_modules", WHERE_GLOBAL );
}


/*    copy_up
                  
      called by : end_of_parse.
                  
      this procedure copyies up all of the need information about a deeper
      level code block into a higher one.  
*/

void copy_up( struct module_info *to, struct module_info *from)
{
  int counter;
  to->lines_of_code += from->lines_of_code;
  to->num_of_comment_lines += from->num_of_comment_lines;
  to->union_lines += from->union_lines;
  to->num_of_uncrowded_statements += from->num_of_uncrowded_statements;
  to->num_of_statements += from->num_of_statements;
  
  for( counter = 0 ; counter < OPERATOR_SIZE ; counter ++ )
    to->operators[counter] += from->operators[counter];
  to->operands_unique += from->operands_unique;
  to->operands_total += from->operands_total;
  
  to->num_of_keywords += from->num_of_keywords;
  
  to->fan_out += from->fan_out;
  to->recursive_calls += from->recursive_calls;
  to->num_of_io_calls += from->num_of_io_calls;
  to->num_of_external_files += from->num_of_external_files;
  
  if( !to->has_horizontal_spacing )
    to->has_horizontal_spacing = from->has_horizontal_spacing;
  if( !to->has_vertical_spacing )
    to->has_vertical_spacing = from->has_vertical_spacing;
  if( !to->is_indented )
    to->is_indented = from->is_indented;
  
  to->num_of_identifiers += from->num_of_identifiers;
  to->num_of_globals_used += from->num_of_globals_used;
  to->num_of_unused_identifiers+= from->num_of_unused_identifiers;
  to->num_of_well_declared_identifiers +=
    from->num_of_well_declared_identifiers;
  to->num_of_initalized_identifiers+=
    from->num_of_initalized_identifiers;
  to->num_of_meaningful_identifiers+=
    from->num_of_meaningful_identifiers;
}


/*   proto
     
     called by : start_proc.
     
     this procedure lookes for a module in the list (LAST_MODULE -> GLOBAL)
     searching for a module with the same name and with the is_prototype
     flag set.  if one is found the fan_in for it is returned and the module
     is discarded. 
*/

int proto()
{
  int fan_in = 0;
  struct module_info *temp = GLOBAL->code_block, *temp_last = GLOBAL;
  
  while(temp != NULL && strcmp(temp->module_name, yyidentifier))
  {
    temp_last = temp;
    temp = temp->code_block;
  }
  
  if(temp != NULL && temp->is_prototype)
  {

    /* DEA '93: Added this check (if statement) to make sure the 
     *          LAST_MODULE pointer does not get "lost" by the 
     *          free_module function. This was happening when parsing 
     *          some prototypes in some code */

    if (LAST_MODULE == temp)
      LAST_MODULE = temp_last;

    temp_last->code_block = temp->code_block;
    fan_in = temp->fan_in;
    free_module(temp);
  }
  
  return(fan_in);
}


/*    start_proc
                 
      called by : yacc.
                 
      This procedure makes a new module.  small bits of information are added
      to ensure that the results will be correct. once the module is made the
      procedure will set the where_in_code flag off of the global spec to the 
      procedure start spec.
*/

void start_proc()
{
  struct module_info *temp;
  struct link_list *list = start_list;
  
  add_declared_identifiers();
  
  temp = malloc_module();
  temp -> code_block = CURRENT_MODULE;
  CURRENT_MODULE = temp;
  
  if(header == 1)
  {
    CURRENT_MODULE->contains_comment_headers = 1;
    header = 0;

    /* DEA '93: The following three line were added to include the header 
     *          comment lines in the "number of comment lines" metrics.
     */

    CURRENT_MODULE->num_of_header_lines += header_lines;
    GLOBAL->lines_of_code -= header_lines;
    GLOBAL->num_of_comment_lines -= header_lines;
  }

  /* DEA '93: The following two lines were added to reset the number of header 
   *          comment lines and union lines.
   */
  
  header_lines = 0;
  CURRENT_MODULE->union_lines = 0;

  temp->fan_in = proto();

  /*  TED '94--added check to see if module starts on first line.
   */
  if(yylineno != 1)
    GLOBAL->lines_of_code--;

  CURRENT_MODULE->lines_of_code++;

  /* lowther93: V(G) must start at 1 at the start of a module */
  count_keywords();
  
  WHERE("start_proc", WHERE_PROTO);
}

/*    prototype
                
      called by : yacc.
                
      This procedure is called when a prototype is found.  If this happens
      the procedure will set the is_prototype flag and add it to the list
      of procedures.  
*/

void prototype()
{
  struct module_info *temp = CURRENT_MODULE;
  struct link_list *list = start_list;

  temp->is_prototype = 1;

  GLOBAL->lines_of_code += temp->lines_of_code;
  GLOBAL->lines_of_code += temp->num_of_header_lines;
  GLOBAL->num_of_comment_lines += temp->num_of_header_lines;

  temp->lines_of_code = 0;
  temp->num_of_header_lines = 0;
  
 /*lowther93: another bug fix from Lanphier. Much check list for Null first. */
  if(list)
  {
    CURRENT_MODULE ->module_name = copy_string(list->information->id_name);
  
    list = list -> next_node;
    free_link(start_list);
    start_list = list;
  }

  while(list)
  {
    list = start_list->next_node;
    free_link(start_list);
    start_list = list;
  }
  start_list = end_list = NULL;
  
  temp->code_block = NULL;
  LAST_MODULE->code_block = temp;
  LAST_MODULE = temp;
  
  CURRENT_MODULE = GLOBAL;
  WHERE("prototype", WHERE_GLOBAL);
}


/*    start_parm_def
           
      called by : yacc.
           
      This procedure sets the where_in_code flag to the parmaeter definition
      section.
*/

void start_parm_def()
{
  WHERE("start_parm_def", WHERE_P_DEF);
}

/*    start_code_block
                  
      called by : yacc.
                  
      this procedure either makes a new code block or in the case of the first
      time through the procedure, it will handle the module name and it's
      parameters.  The procedure will then set the where_in_code flag to 
      the variable declaration section.
*/

void start_code_block()
{
  if(where_in_code == WHERE_I_DEC || where_in_code == WHERE_CODE)
  {
    struct module_info *temp;
    
    statement_list_identifiers();
    
    temp = malloc_module();
    temp -> code_block = CURRENT_MODULE;
    CURRENT_MODULE = temp;
  }
  else
  {
    struct link_list *list = start_list;
     
    CURRENT_MODULE ->module_name = copy_string(list->information->id_name);
    list = list -> next_node;
    free_link(start_list);
    start_list = list;
    
    while(list)
    {
      add_identifier_tree();
      CURRENT_MODULE ->num_of_parameters++;
      CURRENT_MODULE ->num_of_well_declared_identifiers++;
      CURRENT_MODULE ->num_of_initalized_identifiers++;
      
      list = start_list->next_node;
      free_link( start_list);
      start_list = list;
    }
    start_list = end_list = NULL;
  }
  WHERE("start_code_block", WHERE_I_DEC);
}


/*    start_statement
                
      called by : yacc.
                
      this procedure takes care of any left over identifier declarations and 
      sets the where_in_code flag to the statement section.
*/

void start_statement()
{
  add_declared_identifiers();
  WHERE( "start_statement", WHERE_CODE );
}


/*    end_code_block
                 
      called by : yacc.
                 
      This procedure ends a module.  The identifier tree is traversed to it's
      information, the needed information is added up the the prevoius
      module or the module is ended and added to the module list.
*/

void end_code_block()
{
  struct module_info *temp = CURRENT_MODULE;
  
  last_comment_check();

  statement_list_identifiers();
  
  traverse_identifier_tree( temp->identifiers );
  
  CURRENT_MODULE = temp->code_block;
  
  if( CURRENT_MODULE == GLOBAL )
  {
    CURRENT_MODULE = temp;
    traverse_operand_tree( temp->operands ); 
    
    temp->code_block = NULL; 
    LAST_MODULE->code_block = temp;
    LAST_MODULE = temp;

    CURRENT_MODULE = GLOBAL;
    WHERE( "end_code_block", WHERE_GLOBAL );
  }
  else
  {
    copy_up( CURRENT_MODULE, temp );
    WHERE( "end_code_block", WHERE_CODE );
    free_module( temp );
  }
}

/*    end_of_parse
                 
      called by : main.
                 
      This procedure outputs the final information gained by the MAS - CII
      program.  This output will be piped into the back and then into the
      report.
*/

void end_of_parse()
{
  struct module_info *temp, *ftemp;
  
  ftemp = GLOBAL->code_block;
  while( ftemp ) {
    ftemp->fan_in = fan_in_check( ftemp->module_name );
    ftemp = ftemp -> code_block;
  }
  
  LAST_MODULE->code_block = GLOBAL;
  
  temp = GLOBAL->code_block;
  CURRENT_MODULE = temp;
  GLOBAL->code_block = NULL;
  
  while( temp )
  {
    if( temp->is_prototype == 0 )
    {
      
      int counter,total,unique;
      if(temp == GLOBAL )
      {
        traverse_operand_tree( GLOBAL->operands );
        traverse_identifier_tree( GLOBAL->identifiers);
      }
      printf("%c ", temp == GLOBAL ? 'G' : 'M' );

      /* lowther93: limited the function name to 30 letters. */

      printf("%.30s ", temp->module_name );
      printf("%d ", temp->num_of_parameters );

      /*   DEA - 93: This was changed to include the header lines in the LOC
       *   printf("%d ", temp->lines_of_code ); (old version)
       */   

      /*  TED '94--Prints both LOC and NCSS (in that order)
       */
       printf("%d ", temp->lines_of_code + temp->num_of_header_lines ); 
       printf("%d ", temp->lines_of_code - temp->num_of_comment_lines 
	                 + temp->union_lines);


      printf("%d ", temp->num_of_statements );
      printf("%d ", temp->num_of_statements
                    - temp-> num_of_uncrowded_statements );

      /*   DEA - 93: This was changed to include the header lines in the LOC
       *   printf("%d ", temp->num_of_comment_lines); (old version)
       */

      printf("%d ", temp->num_of_comment_lines + temp->num_of_header_lines);

      /* DEA '93:  This was taken out, no longer valid.  This was
       *           just adding one to the # of comment.  Now the 
       *           actually # of header comment lines are added
       *           to "num_of_comment_lines"  
       *
       *      + temp->contains_comment_headers);
       */
 
      printf("%d ", temp->max_nesting );
      
      for(counter = 0, total = 0, unique = 0;
          counter < OPERATOR_SIZE; counter++)
      {
        if(temp->operators[counter])
        {
          unique++;
          total += temp->operators[counter];
        }
      }
      printf("%d ", unique );
      printf("%d ", total );
      
      printf("%d ", temp->operands_unique );
      printf("%d ", temp->operands_total );
      
      /*  lowther93: this doesn't look right...should be v(g)?
       *  This IS v(g) or vsubg etc.
       */

      printf("%d ", temp->num_of_keywords );
            
      printf("%d ", temp->fan_in );
      printf("%d ", temp->fan_out );
      printf("%d ", temp->recursive_calls );
      printf("%d ", temp->num_of_io_calls );
      printf("%d ", temp->num_of_external_files );
      
      printf("%d ", temp->contains_comment_headers );
      printf("%d ", temp->has_horizontal_spacing );
      printf("%d ", temp->has_vertical_spacing );
      printf("%d ", temp->is_indented );
      
      printf("%d ", temp->num_of_identifiers);
      printf("%d ", temp->num_of_globals_used );
      printf("%d ", temp->num_of_unused_identifiers);
      printf("%d ", temp->num_of_well_declared_identifiers);
      printf("%d ", temp->num_of_initalized_identifiers);
      printf("%d ", temp->num_of_meaningful_identifiers);
      printf("%.2f ", (temp->num_of_identifiers) ?
        ((float) temp->average_span /(float) temp->num_of_identifiers)
        : 0);
      
      printf("%d ", temp->max_span);
      printf("\n");
      
    }
    
    temp = temp -> code_block;
    free_module( CURRENT_MODULE );
    CURRENT_MODULE = temp;
  }
  printf("%d\n", num_of_preprocessor_directives );
}

/* DEA and TED - 93: added this to take care of lines of code that
 *                   also has a comment on it.
 */

loc_check(int code_com)
{
  static int last_code_line = 0;
  static int last_com_line = 0;
  static int last_union_line = 0;
  
  if (code_com == CODE)
  {
    if (yylineno == last_com_line && yylineno != last_union_line) 
    {
      CURRENT_MODULE->union_lines++; 
      last_union_line = yylineno;
    }

    last_code_line = yylineno;
  }

  else
    if (code_com == COM)
    {
      if (yylineno == last_code_line && yylineno != last_union_line) 
      {
        CURRENT_MODULE->union_lines++;
        last_union_line = yylineno;
      }
    }
}












