/*$Header: sp-code.h,v 4.0 94/02/01 20:27:45 tedd Exp $*/

#include <stdio.h>
#include <string.h>

#define DEBUG 0  /* PWO */

#define OPERATOR_SIZE 56
#define IO_CALL_SIZE  40
#define EXTERNAL_CALL_SIZE  5

extern int where_in_code;
extern char yytext[];
extern char yyidentifier[];
extern int yyleng;
extern int yylineno;
extern int IS_PROCEDURE_CALL;

#include "where_in_code.h"

typedef int flag;

/* DEA '93:  This structure holds the information of: identifier name, 
 *           first line seen, last line seen, # of times seen, and
 *           if the identifier was initialized. 
 *           
 *           This structure is used by:  struct tree, struct module_info, 
 *           and the struct link_list. */

struct id_info
{
	char *id_name;

	int first_line_seen;
	int last_line_seen;

	int num_of_times_seen;

	flag initialized;
	};

/* DEA '93:  This is a tree structure that holds an identifier information
 *           structure at each node. */

struct tree
{
	struct id_info *information;
	struct tree *left_node;
	struct tree *right_node;
	};

/* DEA '93:  This structure is a link list.  Each node represents a module
 *           (with one node representing the global information) and stores
 *           that modules informations in the struct.  Also each node has 
 *           two trees (identifiers tree and operands tree) to store the 
 *           identifiers and operands information in.  */

struct module_info
{
	char   			*module_name;
	int			is_prototype;
	struct module_info	*code_block;
	struct tree		*identifiers;
	int		num_of_parameters;
	int		lines_of_code;

        /* DEA - 93: Added num_of_header_lines to get accurate count of 
	 *           LOC or NCSS 
	 */
        int             num_of_header_lines;

	int		num_of_comment_lines;

        /* TED - 94: Added union_lines to get accurate count of 
	 *           LOC or NCSS 
	 */
	int             union_lines;
	int		num_of_uncrowded_statements;
	int		num_of_statements;
	int		max_nesting;

	int		operators[ OPERATOR_SIZE ];
	struct tree	*operands;
	int		operands_unique;
	int		operands_total;

	int		num_of_keywords;   /*    V(G)   */

	int		fan_in;
	int		fan_out;
	int		recursive_calls;
	int		num_of_io_calls;
	int		num_of_external_files;

	int 		num_of_identifiers;
	int		num_of_globals_used;
	int 		num_of_unused_identifiers;
	int		num_of_well_declared_identifiers;
	int		num_of_initalized_identifiers;
	int		num_of_meaningful_identifiers;
	float		average_span;
	int		max_span;

	flag		is_indented;
	flag		contains_comment_headers;
	flag		has_horizontal_spacing;
	flag		has_vertical_spacing;
};

extern int num_of_preprocessor_directives;

/* DEA '93:  This structure is a link list.  The link list is used to store 
 *           the tokens in one line of code.  The link list is then traversed
 *           to find the identifiers.  As far as I can tell *user_list is 
 *           never used!  */

struct link_list
{
	struct id_info *information;
	struct link_list *next_node;
};

/*  TED '94--Function prototypes for functions that were previously
 *	     defined in this header file but are now in common.c.
 */

struct module_info *malloc_module( );
void free_module( struct module_info *temp );
struct id_info *malloc_id();
void free_id( struct id_info *temp );
struct link_list *malloc_link();
void free_link( struct link_list *temp );
struct tree *malloc_tree();
void free_tree( struct tree *temp );
char *copy_string( char *string );
char *yyid_copy();
void last_comment_check(void);
