
/*********************************************************************
*	 Maintainability Analysis Report Generator                   *
*                                                                    *
*Author : Regi John                                                  *
*Date   : 10/2/94                                                    * 
*Description : This is ver 2.0 of hpmet. This version is to obtain   *
*	       the metrics data from the report files of both        *
*	       UXMETRIC and PCMETRIC. For the .rpt/rp1/rp2 files     *
*              the program first scans the report file and finds out *
*              the relative offsets for the required metrics. The 3  *
*              or 4 metric formula is used depending on the type of  *
*              report.						     *	
*********************************************************************/

#include "struct.h"
#include "fproto.h"

#define clear_array(x,n) {int i; for (i = 0;i < n-1;i++) x[i] = ' '; x[i]='\0';}

/******** Global variables **********************************/
int offset[4]={0};
int fld_size[4]={0};
int cmtflg=1;
/***********************************************************/

int is_not_blank(char [],int);

void main(int argc, char **argv )
{
   FILE *ipf=NULL;
   char file_name[MAX_FILE_NAME_SIZE] = {' '};
   char *err_msg,filetype;
   int file_ok=0;
   FILENODE *fnode;
   int num_of_arg,num_of_file;
 
   if (argc == 1)
   {
      err_msg_disp("Usage : micalc sourcefile ... [> targetfile]");
      exit(1);
    }


   num_of_arg = argc - 1;
   num_of_file = 1;

   print_header();
   while (num_of_arg > 0)
   {
   strcpy(file_name,argv[num_of_file]);
   ipf = fopen(file_name,"r");

   if (ipf == NULL)
   {
      err_msg_disp("File not found"); 
      exit(1);
   }
   else
      filetype = get_file_type(ipf);

   printf("Report Name : %s\n\n",file_name);

   if (filetype == 'r')
      get_file_data(ipf);
   else
   {
      if (filetype == 'p')
	 prn_file_proc(ipf);
      else
      {
	 err_msg_disp("Error while trying to find filetype");
	 err_msg_disp("Aborting");
	 exit(1);
      }
    } 

   print_tail();
   num_of_arg --;
   num_of_file ++;

   close(ipf);
   }
}

/****************************************************************
* Function name : get_file_type					*
* Description : This function finds out if the input file is a  *
*		.rpx or .prn file. It does this by checking if  *
*		the first character of the file. A .prn file has*
*		a '"' as its first character			*
****************************************************************/
char get_file_type(FILE *fl)
{
   char cbf[BUFSIZ]={" "};

   fgets(cbf,BUFSIZ,fl);

   if (cbf[0] == '\"')
   {
      fseek(fl,0L,SEEK_SET);
      return (PRN);
   }
   else
   {
      fseek(fl,0L,SEEK_SET);
      return (RPX);
   }

}

/*************************************************************
* Function name : get_file_data				     *
* Description : This module builds up a 2d linked list with  *
*		the data from the input file. A FILENODE     *
*               points to a linked list of MODENODEs contain-*
*		ing data for a particular file               *
*							     *	
*************************************************************/
void get_file_data(FILE *infile)
{
   FILENODE *fl_head=NULL,*fl_tail=NULL,*fl_temp=NULL;
   MODNODE *m_head=NULL,*m_tail=NULL,*m_temp=NULL;
   int fst_time=1,i;
   double prcmt=0.0;
   char fl_buf[BUFSIZ]={" "};

   fl_head = malloc(sizeof(FILENODE));
   fl_tail = fl_head;

   while (!end_of_report(infile,fl_buf))
   {  
      init_filenode(fl_tail);

      get_file_name(fl_tail,fl_buf);
      if (!fst_time)
      {
         if ((strcmp(fl_temp->file_name,fl_tail->file_name) != 0))
            m_head = NULL;
         else
         {
            fl_tail = fl_temp;
            fl_tail->next_file = NULL;
         }
      }

      fst_time = 0;
      get_offset(infile);

      clear_array(fl_buf,BUFSIZ);
      fgets(fl_buf,BUFSIZ,infile);
      while (fl_buf[0] != 12 && is_not_blank(fl_buf,BUFSIZ))
      {
         if (m_head == NULL)
         {
            m_head = malloc(sizeof(MODNODE));
            m_tail = m_head;
         }
         else
         {
            m_tail->next = malloc(sizeof(MODNODE));
            m_tail = m_tail->next;
         }

         m_tail->next = NULL;

         for (i=0;i<4;i++)
            m_tail->mod_data[i] = get_data(fl_buf,offset[i],fld_size[i]);

         if (m_tail->mod_data[2] == 0)
	 {
	    fprintf(stderr,"For file : %s\n",fl_tail->file_name);
	    err_msg_disp("Unable to calculate percentage comment");
	    err_msg_disp("This could be due to lines of code = 0");
	    err_msg_disp("Aborting");
	    exit(1);
         }

         prcmt = ((double) (m_tail->mod_data[3]))/m_tail->mod_data[2];
         m_tail->mi = m_index(prcmt,m_tail->mod_data[0],m_tail->mod_data[1],
                              m_tail->mod_data[2]);
         fl_tail->no_of_modules += 1;
         fl_tail->total_e += m_tail->mod_data[0];
         fl_tail->total_vg2 += m_tail->mod_data[1];
         fl_tail->total_loc += m_tail->mod_data[2];
         fl_tail->total_cmt += m_tail->mod_data[3];
   
         clear_array(fl_buf,BUFSIZ);
         fgets(fl_buf,BUFSIZ,infile);
      }

      fl_tail->mod_node = m_head;
      fl_tail->next_file = malloc(sizeof(FILENODE));
      fl_temp = fl_tail;
      fl_tail = fl_tail->next_file;

   }

   fl_tail = fl_temp;
   fl_tail->next_file = NULL;

   fl_temp = fl_head;

   cal_mi(fl_head);
}

/****************************************************************
* Function name : is_not_blank					*
* Description : This function checks to see if a line read from * 
*		the report file is blank or not. A blank line   *
*		signifies the end of a report 			*
****************************************************************/
int is_not_blank(char strbuf[],int b_size)
{
  int i,flg;
  char c;

  for (i=0;i<1;i++)
  {
     c=strbuf[i];
     if (isspace(c))
       flg=0;
     else
       flg=1;
  }

  return flg;
}



/*************************************************************
* Function name : end_of_report				     *
* Description : This module checks if the end of the report  *
*		is reached by scanning for the string        *
*		"Summary".				     *
*							     *
*************************************************************/
int end_of_report(FILE *flptr, char ipstr[])
{   
      char bln_arr[BUFSIZ]={" "};

      while (fgets(ipstr,BUFSIZ,flptr) != NULL)
      {
      if (strncmp(ipstr,"Complex",7) == 0)
         return 0;
      else
         if (strncmp(ipstr,"Summary",7) == 0)
	    return 1;
      }

      err_msg_disp("This is not an .rpt or .rpx file");
      err_msg_disp("Aborting");
      exit(1);
}

/* This function initialises a FILENODE */
void init_filenode(FILENODE *fnodex)
{
   fnodex->total_e = 0;
   fnodex->total_vg2 = 0;
   fnodex->total_loc = 0;
   fnodex->total_cmt = 0;
   fnodex->no_of_modules = 0;
}


/*************************************************************
* Function name : set_pointer                                *
* Description : This procedure skips "skip_lines" of the     *
*               file pointed by *ptr                         *
*                                                            *
*************************************************************/
void set_pointer(FILE *ptr,int skip_lines)
{
   char c;
   int i;

   for (i = 1;i <= skip_lines;i++)
      while ((c = getc(ptr)) != '\n');
}

/****************************************************************
* Function name : get_file_name                                 *
* Description : This module extracts the name of the file whose *
*		metrics are being scanned		        *
*								*
****************************************************************/
void get_file_name(FILENODE *fnd,char cbuf[])
{
   int i=0,j=0;

   while (cbuf[i] != ':') i++;
   i++;

   while (cbuf[i] != '\n' && j <= MAX_FILE_NAME_SIZE)
   {
      fnd->file_name[j] = cbuf[i];
      i++;
      j++;
   }

}

/****************************************************************
* Function name : get_offset					*
* Description : This function obtains the relative offsets for  *
* 		E,VG2,LOC,CMT in the input report file          *
****************************************************************/
void get_offset(FILE *f_ptr)
{
   char ch_buf[BUFSIZ]={" "};
   char cmbuf[4];
   int start_ptr=0,end_ptr=0;
   int i=0,field_size=0;
   int foundcmt=0;

   do
   {
      if (fgets(ch_buf,BUFSIZ,f_ptr) == NULL)
      {
	 err_msg_disp("End of file reached. Aborting\n");
	 exit (1);
      }

   }
   while ((strncmp(ch_buf,"Procedure",(size_t) 9) != 0) &&
	  (strncmp(ch_buf,"Function",(size_t) 8) != 0)); 

   while (ch_buf[end_ptr] != '\n' && end_ptr < BUFSIZ)
   {
     start_ptr = end_ptr;

     while (ch_buf[end_ptr] == ' ' && end_ptr < BUFSIZ) 
     end_ptr ++;
      
     clear_array(cmbuf,4);

     i = 0;
     while (ch_buf[end_ptr] != ' ' && i < 3 && end_ptr < BUFSIZ) 
     {
	cmbuf[i] = ch_buf[end_ptr];
	i++;
	end_ptr++;
     }

     if (ch_buf[end_ptr] != ' ')
     {
	clear_array(cmbuf,3);
	while (ch_buf[end_ptr] != ' ' && ch_buf[end_ptr] != '\n') end_ptr++;
     }

     field_size = end_ptr - start_ptr - 1;

     if (strcmp (cmbuf,E) == 0)
     {
	   offset[0] = start_ptr + 1;
	   fld_size[0] = field_size;
     }
     else 
	if (strcmp(cmbuf,VG2) == 0)
	{
	   offset[1] = start_ptr + 1;
	   fld_size[1] = field_size;
        }
	else
	   if ((strcmp(cmbuf,LOC) == 0) || (strcmp(cmbuf,LOC4) == 0))
	   {
	      offset[2] = start_ptr + 1;
	      fld_size[2] = field_size;
           }
	   else 
	      if ((strcmp(cmbuf,CMT) == 0) || (strcmp(cmbuf,CMT4) == 0))
	      {
	         offset[3] = start_ptr + 1;
	         fld_size[3] = field_size;	
		 foundcmt++;
              }
     
   }

   if (!foundcmt)
   {
      offset[3] = 0;
      cmtflg=0;
   }
   else
      cmtflg = 1;
   set_pointer(f_ptr,1);
}


/******************************************************************
* Function name : get_data					  *
* Description : This procedure accesses the data in the report    *
*		file and returns its integer value. The parameters*
*		are the pointer to the buffer, the offset of the  *
*		required data and the size of the data field      *
*								  *
******************************************************************/
int get_data(char data_buf[],int offset, int size)
{
   char temp_buf[10] = {" "};
   int i=0;

   for (i = 0;i < size;i++)
      temp_buf[i] = data_buf[offset + i];

   return (atoi(temp_buf));
 
}

/*****************************************************************
* Function name : err_msg_disp					 *
* Description : The procedure displays error messages		 *
*								 *
*****************************************************************/
void err_msg_disp(char *msg)
{
   fprintf(stderr,"%s\n",msg);
}

/*****************************************************************
* Function name : m_index                                        *
* Description : This function calculates the maintainability     *
*               index based on E,VG,LOC,CMT                      *
*                                                                *
*****************************************************************/
int m_index(double percmt,int he,int vg,int loc)
{
   double mi=0.0;

      mi = 171 - (3.42 * log((double) he)) - (0.23 * ((double) vg))
	      - (16.2 * log((double) loc)) + (50 * sin(sqrt(2.46 * percmt)));

       if (mi > 0)
       return (int) (mi + 0.5);
       else
	return (int) (mi - 0.5);
}

/*****************************************************************
* Function name : cal_mi				         *
* Description : This file calculates the percentage of modules   *
*		with MI less than 65, between 65 and 85 and      *
*		greater than 85					 *
*								 *
*****************************************************************/
void cal_mi(FILENODE *f_ptr)
{
   MODNODE *mod_ptr=NULL;
   FILENODE *file_ptr=NULL;
   int mi_ar[3]={0,0,0};
   int loc_ar[3]={0,0,0};
   int sys_mi_ar[3]={0,0,0};
   int sys_loc_ar[3]={0,0,0};
   int tot_mod=0,tot_loc=0;
   int sys_tot_mod=0,sys_tot_loc=0;
   int mod_mi=0,mod_loc=0;
   int i;
   int avg_sys_mi=0;

   file_ptr = f_ptr;
   while (file_ptr != NULL)
   {
      mod_ptr = file_ptr->mod_node;
      tot_mod = file_ptr->no_of_modules;
      tot_loc = file_ptr->total_loc;
      sys_tot_mod += tot_mod;
      sys_tot_loc += tot_loc;

      for (i = 0;i < 3;i++)
      {
	 mi_ar[i] = 0;
	 loc_ar[i] = 0;
      } 

      while (mod_ptr != NULL)
      {
	 mod_mi = mod_ptr->mi;
	 mod_loc = mod_ptr->mod_data[2];

	 if (mod_mi < 65)
	 {
	    mi_ar[0]++;
	    loc_ar[0] += mod_loc;
	    sys_mi_ar[0] ++;
	    sys_loc_ar[0] += mod_loc;
         }
         else
	 {
	    if ((mod_mi >= 65) && (mod_mi <= 85))
	    {
	       mi_ar[1] += 1;
	       loc_ar[1] += mod_loc;
	       sys_mi_ar[1] += 1;
	       sys_loc_ar[1] += mod_loc;
	    }	
	    else
	    {
	       mi_ar[2] += 1;
	       loc_ar[2] += mod_loc;
	       sys_mi_ar[2] += 1;
	       sys_loc_ar[2] += mod_loc;
	    }
	 }



	 mod_ptr = mod_ptr->next;
      }
      file_ptr->file_mi = calfile_mi(*file_ptr);
      
      print_data(file_ptr->file_name,file_ptr->file_mi,mi_ar,loc_ar,tot_mod 
                 ,tot_loc);

      file_ptr = file_ptr->next_file;
   }
   avg_sys_mi = calsys_mi(f_ptr);
   print_data("Total System",avg_sys_mi,sys_mi_ar,sys_loc_ar,sys_tot_mod,sys_tot_loc);

   if (cmtflg == 0)
      printf("\n\n* :  3-metric model is used");

}


/*****************************************************************
* Function name : calfile_mi					 *
* Description : This function calculates the MI of the file.     *
*               Average E, VG2, LOC and CMT are used for the     *
*               calculation					 *
*								 *
*****************************************************************/
int calfile_mi(FILENODE flnode)
{
   int avg_e=0,avg_vg2=0,avg_loc=0,avg_cmt=0;
   int no_of_mod=0;
   double prccmt=0.0;
   int f_mi=0;

   no_of_mod = flnode.no_of_modules;

   prccmt = ((double) flnode.total_cmt)/flnode.total_loc;
   avg_e = ((((double) flnode.total_e)/no_of_mod) + 0.5);
   avg_vg2 = ((((double) flnode.total_vg2)/no_of_mod) + 0.5);
   avg_loc = ((((double) flnode.total_loc)/no_of_mod) + 0.5);

   f_mi = m_index(prccmt,avg_e,avg_vg2,avg_loc);
   return(f_mi);
}


/*****************************************************************
* Function name : calsys_mi					 *
* Description : This function calculates the average MI for the  *
*               whole system. This is done by traversing thru the*
*               the file nodes.					 *	
*								 *	
*****************************************************************/
int calsys_mi(FILENODE *file_p)
{
   FILENODE *temp_ptr=NULL;
   int total_modules=0;
   int tot_sys_e=0,tot_sys_vg2=0,tot_sys_loc=0,tot_sys_cmt=0;
   int avg_sys_e=0,avg_sys_vg2=0,avg_sys_loc=0,avg_sys_cmt=0;
   int tot_sys_mod=0;
   double sys_per_cmt=0.0;
   int s_avg_mi=0;

   temp_ptr = file_p;

   do
   {
      tot_sys_e += temp_ptr->total_e;
      tot_sys_vg2 += temp_ptr->total_vg2;
      tot_sys_loc += temp_ptr->total_loc;
      tot_sys_cmt += temp_ptr->total_cmt;
      tot_sys_mod += temp_ptr->no_of_modules;

      temp_ptr = temp_ptr->next_file;

   }
   while(temp_ptr != NULL);


   avg_sys_e = ((((double) tot_sys_e)/tot_sys_mod) + 0.5);
   avg_sys_vg2 = ((((double) tot_sys_vg2)/tot_sys_mod) + 0.5);
   avg_sys_loc = ((((double) tot_sys_loc)/tot_sys_mod) + 0.5);
   sys_per_cmt = ((double) (tot_sys_cmt))/tot_sys_loc;

   s_avg_mi = m_index(sys_per_cmt,avg_sys_e,avg_sys_vg2,avg_sys_loc);

   return (s_avg_mi);
}
/*****************************************************************
* Function name : prcntge					 *
* Description : This function calculates the percentage of the 2 *
*		numbers that are input.				 *
*								 *
*****************************************************************/
int prcntge(int num1, int num2)
{
   return (int) ((((double) num1/(double) num2) * 100) + 0.5);
}


/*****************************************************************
* Function name : print_header					 *
* Description : This function prints the header for the output   *
*								 *
*****************************************************************/
void print_header(void)
{
   printf("\n\n\n\n");
   printf("\t\t\t");
   printf("Maintainability Index Report\n");
   printf("\t\t-----------------------------------------\n");
   printf("\t\t\t\t\t\t\t");
   printf("%%<65  65<=%%<=85  %%>85\n");
}


/*****************************************************************
* Function name : print_data					 *
* Description : This procedure prints the MI data for each module*
*								 *
*****************************************************************/
void print_data(char *lin_ch,int fl_mi, int ar1[],int ar2[],int totmd,int totloc)
{
   int i;

   printf("File :\t%s\n",lin_ch);
   
   if  (cmtflg == 0)
      printf("\t\b*Avg. MI = %3d", fl_mi); 
   else
      printf("\tAvg. MI = %3d", fl_mi); 

   printf("\t\t\tBy module :");
   for (i = 0;i < 3;i++)
      printf("\t%3d",prcntge(ar1[i],totmd));

   printf("\n");

   printf("\t\t\t\t\tBy LOC :");
   for (i = 0;i < 3;i++)
      printf("\t%3d",prcntge(ar2[i],totloc));     

   printf("\n\n");
}

void print_tail(void)
{
	printf("\n");
	printf("\t\t------------------------ X ------------------------------\n\n\n");
}
